/*-----------------------------------------------------------------------+
|             DEPARTAMENTO MECANICA UNIVERSIDADE AVEIRO                  |
|                                                                        |
| Nome:                                                                  |
|       David Manuel Costa Gameiro          N:20281                      |
|       Filipe Carvalho Mostardinha         N:22085                      |
|                                                                        |
| Data:   10/Maio/2004     Nome Ficheiro: InitCan.c                      |
+-----------------------------------------------------------------------*/

#include "can.h"
#include "usart.h"
#include "Can18xx8.h"

// Comprimento das mensagens CAN
#define CAN_MESSAGE_LENGTH   8

// Alguns bytes das mensagens CAN (de recepo)
#define CAN_TYPE    0
#define CAN_DATA1   1

// _____ IDENTIFICADOR DAS MENSAGENS CAN ________________________________________________________________
// Constantes de referncia ao identificador das mensagens CAN
enum MESSAGE_IDENTIFIER  {
    // Mscaras dos campos
    OPERATION_MASK = 0b11000000000,     // 0b11xxxxxxxxx: Operao
    XCUSRC_MASK    = 0b00011110000,     // 0bxxx1111xxxx: Unidade de controlo origem
    XCUDST_MASK    = 0b00000001111,     // 0bxxxxxxx1111: Unidade de controlo destino

    // Localizao dos campos
    OPERATION_LOC  = 9,
    XCUSRC_LOC     = 4,
    XCUDST_LOC     = 0,

    // Operaes (bits 0b11xxxxxx)
    OP_UPDATE_SENSORS   = 0b00,         // Actualizao dos sensores
    OP_UPDATE_ACTUATORS = 0b01,         // Actualizao dos actuadores
    OP_SLAVECAN_STATUS  = 0b10          // Estado do barramento CAN
};

// Extrao dos campos do identificador
#define operation(x) (((unsigned int) (x) & OPERATION_MASK) >> OPERATION_LOC)
#define xcuSRC(x)    (((unsigned int) (x) & XCUSRC_MASK   ) >> XCUSRC_LOC   )
#define xcuDST(x)    (((unsigned int) (x) & XCUDST_MASK   ) >> XCUDST_LOC   )

// Construo de padres do identificador
#define operation_patt(y) (((unsigned int) (y) << OPERATION_LOC) & OPERATION_MASK)
#define xcuSRC_patt(y)    (((unsigned int) (y) << XCUSRC_LOC   ) & XCUSRC_MASK   )
#define xcuDST_patt(y)    (((unsigned int) (y) << XCUDST_LOC   ) & XCUDST_MASK   )

// Construo do identificador
#define identifierMSG(op,dst) (operation_patt(op) | xcuSRC_patt(HOST_MCU) | xcuDST_patt(dst))
// ____________________________________________________________________________________________________

// _____ PACOTE DE TIPO DAS MENSAGENS OP_UPDATE_SENSORS _______________________________________________
// Constantes de referncia ao byte de tipo das mensagens de actualizao de sensores (operao)
enum MESSAGE_UPDATE_SENSORS  {
    // Mscaras para extrao de um campo
    SCUSRC_MASK   = 0b11110000,         // 0b1111xxxx: id do scu fonte
    MSGINDEX_MASK = 0b00001000,         // 0bxxxx1xxx: ndice da mensagem de actualizao dos sensores
    SPECIAL_MASK  = 0b00000100,         // 0bxxxxx1xx: Existem sensores adicionais?
    SENSORS_MASK  = 0b00000010,         // 0bxxxxxx1x: Tipos de sensores
    FOOTID_MASK   = 0b00000001,         // 0bxxxxxxx1: ID do p (se houver)

    // Localizao dos campos no byte de tipo
    SCUSRC_LOC    = 4,
    MSGINDEX_LOC  = 3,
    SPECIAL_LOC   = 2,
    SENSORS_LOC   = 1,
    FOOTID_LOC    = 0,

    // Identificao do pacote
    MSGINDEX_1 = 0,                     // Primeira mensagem
    MSGINDEX_2 = 1,                     // Segunda mensagem

    // Conjuntos de sensores (identificado pelo bit 0bxxxxxx1x
    SENSORS_REACTION    = 0,
    SENSORS_ORIENTATION = 1
};

// Macros para leitura dos diversos campos no byte de tipo
#define scuSRC(x)   (((unsigned int) (x) & SCUSRC_MASK  ) >> SCUSRC_LOC  )
#define msgIndex(x) (((unsigned int) (x) & MSGINDEX_MASK) >> MSGINDEX_LOC)
#define special(x)  (((unsigned int) (x) & SPECIAL_MASK ) >> SPECIAL_LOC )
#define sensors(x)  (((unsigned int) (x) & SENSORS_MASK ) >> SENSORS_LOC )
#define footID(x)   (((unsigned int) (x) & FOOTID_MASK  ) >> FOOTID_LOC  )
// ___________________________________________________________________________________________________

// _____ PACOTE DE ESTADO DO SISTEMA _________________________________________________________________
// Pacote de estado do sistema (ltimo byte das primeira mensagem OP_UPDATE_SENSORS
enum MESSAGE_SYSSTATUS_PACKET {
    // Mscaras dos bits
    DEADLINEERROR_MASK = 0b00000100,     // Ocorrncia de violao de deadline
    CONTROLON_MASK     = 0b00000010,     // Controlador PID ligado
    RESET_MASK         = 0b00000001,     // Motores nas home positions

    // Localizao dos bits
    DEADLINEERROR_LOC  = 2,
    CONTROLON_LOC      = 1,
    RESET_LOC          = 0
};

// Extrao dos bits do pacote de estado do sistema
#define deadlineError(x)    (((unsigned int) (x) & DEADLINEERROR_MASK) >> DEADLINEERROR_LOC)
#define controlOn(x)        (((unsigned int) (x) & CONTROLON_MASK    ) >> CONTROLON_LOC    )
#define reset(x)            (((unsigned int) (x) & RESET_MASK        ) >> RESET_LOC        )
// ___________________________________________________________________________________________________

// _____ PACOTE DE TIPO DAS MENSAGENS OP_SLAVECAN_STATUS _____________________________________________
// Estados de erro relativos ao CAN
enum CAN_ERRORS  {
    TX_FAILED           = 0b10000000,       // Transmisso falhada
    RX_OVERFLOW_ERROR   = 0b00000001,       // Erro de overflow dos buffers de recepo
    RX_INVALIDMSG_ERROR = 0b00000010,       // Mensagem invlida
    RX_INVALID_LENGTH   = 0b00000100,       // Comprimento invlido
    RX_INVALID_DATA     = 0b00001000,       // Dados da mensagem invlidos
    RX_INVALID_OPERATION= 0b00010000,       // Operao invlida
    RX_UNKNOWN_ERROR    = 0b00100000,       // Erro desconhecido

    TX_BITS = 0b10000000,                   // Bits relativos  transmisso
    RX_BITS = 0b01111111                    // Bits relativos  recepo
};
// ___________________________________________________________________________________________________

// Estrutura descritiva dos erros ocorridos
struct  {
    byte flags;                     // Causas
    byte tx_count, rx_count;        // Nmero de ocorrncias
} canErrors={false,0,0};

static bool updateActuators(byte scu_id, byte *can_message);// Construo de uma mensagem para actualizao dos actuadores
static bool updateSensors(byte scu_id, byte *can_message);  // Actualizao dos dados sensoriais do sistema
static void setMcuError(enum CAN_ERRORS error);             // Registo de um erro ocorrido com o CAN
static void setScuError(byte scu_id, byte *can_message);       // Transferncia de erros de um SCU para o MCU


// ****************************************************************************
//Macros relativas ao CAN, todos os valores sao obtidos tendo em conta um
//um baud rate de 1 Mbps
#define FASE_SYNC 2	    //	Time Quantums para a fase de sincronismo
#define FASE_PROP 2		//	Time Quantums para a fase de propagao
#define FASE_SEG1 2		//	Time Quantums para o segmento 1
#define FASE_SEG2 2		//	Time Quantums para o segmento 2
#define BRP       3 	//  Baud Rate (833.3 Kbps)

// Mascara autorizando a que o filtro verifique apenas os 4 bits menos
// significativos (SCU ID)
#define MASCARA  XCUDST_MASK

//o byte ob1abcdefg , definido da seguinte forma
// a,b - Receive Buffer Mode bits
// c - Receive Buffer 0 Double-buffer enable bit
// d - qual o identificador a usar (1) standard (0) expandHostHostIDo
// e - amostrar 3 vezes o Bus antes do ponto amostra
// f - acordar o CAN atrav,s do bus, (1) Sim  (0) Nao
// g - Segmento 2 baud rate programado livremente ou nao
#define CONFIGURACAO 0b10101001 //O tipo de configuracao desejada

// Configurao do Timer2 para transmisso peridica de mensagens CAN
#define TIMER2_PRES 16
#define TIMER2_POST 8
#define TIMER2_OUT  3200
#define TIMER2_PER  (byte) ((unsigned int) TIMER2_OUT*F_CPU/((unsigned int) TIMER2_PRES*TIMER2_POST)-1)

// Inicializao do CAN
//
void canInit(void)
{
    // Programacao dos pinos RB3 e RB2 como entrada e sada respectivamente
	TRISB=(TRISB | 0x08) & 0xFB;

	// Colocar o MCU em modo configuracao relativamente ao CAN
	CANSetOperationMode(CAN_OP_MODE_CONFIG);

    // Inicializao do Baud Rate do CAN
	CANInitialize(FASE_SYNC, BRP, FASE_SEG1,
				  FASE_SEG2, FASE_PROP, CONFIGURACAO);

	//Definicao das Mascaras e Filtros
	CANSetMask(CAN_MASK_B1, MASCARA, CAN_CONFIG_STD_MSG);
	CANSetMask(CAN_MASK_B2, MASCARA, CAN_CONFIG_STD_MSG);

	CANSetFilter(CAN_FILTER_B1_F1, HOST_MCU, CAN_CONFIG_STD_MSG);
	CANSetFilter(CAN_FILTER_B1_F2, HOST_MCU, CAN_CONFIG_STD_MSG);

	CANSetFilter(CAN_FILTER_B2_F1, HOST_MCU, CAN_CONFIG_STD_MSG);
	CANSetFilter(CAN_FILTER_B2_F2, HOST_MCU, CAN_CONFIG_STD_MSG);
	CANSetFilter(CAN_FILTER_B2_F3, HOST_MCU, CAN_CONFIG_STD_MSG);
	CANSetFilter(CAN_FILTER_B2_F4, HOST_MCU, CAN_CONFIG_STD_MSG);

	//Activar o modo Normal de Funcionamento
	CANSetOperationMode(CAN_OP_MODE_NORMAL);

    // Activa os interrupts dos buffers recepcao CAN
    PIE3bits.RXB0IE=1;                                  // Interrupts Enable
    PIE3bits.RXB1IE=1;


    // Configurao do Timer2 para gerao de interrupes de 1000 em 1000 us
    // Este interrupt ser usado para transmisses de mensagens CAN
    T2CONbits.TOUTPS3=0;
    T2CONbits.TOUTPS2=1;
    T2CONbits.TOUTPS1=1;
    T2CONbits.TOUTPS0=1;        // Postscale 1:8

    T2CONbits.T2CKPS1=1;        // Prescale 1:16
    T2CONbits.T2CKPS0=0;

    PR2=TIMER2_PER;             // Definio do perodo
    TMR2=0;                     // Inicializao do contador
    T2CONbits.TMR2ON=1;         // Timer2 ON
    PIE1bits.TMR2IE=1;          // Activao das interrupes

    // Activao das interrupes globais, no mduloPIC.H/C
}

// ****************************************************************************
/////////////////////////////////////////////////////////////////////////
// Constante de configuracao do envio de mensagens                     //
//																	   //
// byte definido 0b1a111bcd											   //
// a - pretende-se notificacao de recepcao de msg (1) caso contrario 0 //
// b - indica qual o tipo de indentificador (1)STD (0)XTD			   //
// [c,d] - prioridade a dar a esta mensagem, ver registo TXBnCON	   //
/////////////////////////////////////////////////////////////////////////

#define CAN_TX_FLAGS 0b11000111

// Transmisso de uma mensagem CAN relativa  actualizao dos actuadores do sistema
//
bool canSendMsg(byte scu_id)
{
    bool res;                           // Resultado da transmisso
    byte message[CAN_MESSAGE_LENGTH];   // Mensagem CAN
    unsigned int identifier;            // IDentificador da mensagem

    // Construo da mensagem a enviar
    if (!updateActuators(scu_id,message))  return false;

    // Identificador da mensagem
    identifier=identifierMSG(OP_UPDATE_ACTUATORS,scu_id);

    // Envio da mensagem
    if (!(res=CANSendMessage(identifier, message, CAN_MESSAGE_LENGTH, CAN_TX_FLAGS)))
        setMcuError(TX_FAILED);     // Caso o envio falhe, notificar o erro ao PC

    // Retorno do sucesso/falha da transmisso
    return res;
}


// ****************************************************************************
// Recepo de uma mensagem CAN de um dos SCU, relativa  actualizao dos sensores
//
bool canReceiveMsg(void)
{
    unsigned long identifier;                       // Identificador da mensagem recebida
    byte length;                                    // Comprimento, em bytes, da mensagem
    enum CAN_RX_MSG_FLAGS warnings;                 // Avisos sobre a recepo
    byte message[CAN_MESSAGE_LENGTH];               // Mensagem CAN recebida

    // Um erro ocorreu na recepo?
    if (!CANReceiveMessage(&identifier, message, &length, &warnings))
        return false;

    // Erro de Overflow
    if (warnings & CAN_RX_OVERFLOW)  {
        setMcuError(RX_OVERFLOW_ERROR);
  		COMSTAT &= 0b00111111;                  // Overflow state clear
    }
    // Erro de mensagem invlida
    else if (warnings & CAN_RX_INVALID_MSG)  {
        setMcuError(RX_INVALIDMSG_ERROR);
  		PIR3bits.IRXIF = 0;                     // Invalid Message state clear
    }

    // Comprimento invlido do pacote
    if (length<CAN_MESSAGE_LENGTH)  {
        setMcuError(RX_INVALID_LENGTH);
    }
    // Divulgao do status do SCU origem
    else if (operation(identifier)==OP_SLAVECAN_STATUS)
        setScuError(xcuSRC(identifier),message);

    // Tudo OK: Actualizao dos sensores
    else if (operation(identifier)==OP_UPDATE_SENSORS)
        updateSensors(xcuSRC(identifier),message);

    // Erro desconhecido
    else
        setMcuError(RX_UNKNOWN_ERROR);

    return true;
}

// ****************************************************************************
// Construo da mensagem a enviar ao slave para actualizao dos actuadores
//
static bool updateActuators(byte scu_id, byte *can_message)
{
    byte i;
    byte scu;

    // Clculo do ndice dos arrays associados ao scu_id,
    // e sua validao
    if ((scu=scuIndex(scu_id))>=N_SCU)  {
        setMcuError(RX_INVALID_DATA);
        return false;
    }

    // Preenchimento do pacote a enviar
    for (i=0; i<N_SERVOS; ++i)  {
        can_message[i]=actuators.servo[scu][i].position;
        can_message[N_SERVOS+i]=actuators.servo[scu][i].velocity;
    }
    // Parmetros do controlador
    can_message[2*N_SERVOS+0]=actuators.control.kp;
    can_message[2*N_SERVOS+1]=actuators.control.kd;

    // Finalizao do preenchimento com zeros
    for (i=2*N_SERVOS+2; i<CAN_MESSAGE_LENGTH; ++i)  can_message[i]=0;

    return true;
}

// ****************************************************************************
// Actualizao dos dados sensoriais do sistema
//
static bool updateSensors(byte scu_id, byte *can_message)
{
    byte i;
    byte scu, foot;
    byte msg_type=can_message[CAN_TYPE];

    // Validao do SCU e do p, se solicitado
    if ((scu=scuIndex(scu_id))>=N_SCU || (foot=footID(msg_type))>=N_FEET)  {
        setMcuError(RX_INVALID_DATA);
        return false;
    }

    // Se o pacote recebido corresponde ao primeiro...
    if (msgIndex(msg_type)==MSGINDEX_1)  {
        // Actualizar as posies e velocidades dos servos da junta
        for (i=0; i<N_SERVOS; ++i)  {
            sensors.servo[scu][i].position=can_message[CAN_DATA1+i];
            sensors.servo[scu][i].velocity=can_message[CAN_DATA1+N_SERVOS+i];
        }
        // Actualizao do estado do sistema
        sensors.sysStatus.reset=reset(can_message[CAN_DATA1+2*N_SERVOS]);
        sensors.sysStatus.controlOn=controlOn(can_message[CAN_DATA1+2*N_SERVOS]);
        sensors.sysStatus.deadlineError=deadlineError(can_message[CAN_DATA1+2*N_SERVOS]);
    }
    // Se o pacote recebido corresponde ao segundo (ltimo)...
    else  {
        // Actualizar as correntes consumidas pelos servos da junta
        for (i=0; i<N_SERVOS; ++i)
            sensors.servo[scu][i].current=can_message[CAN_DATA1+i];

        // Existem sensores adicionais?
        if (special(msg_type))  {
            // Se a informao sensorial corresponde aos sensores de fora, actualiz-los
            if (sensors(msg_type)==SENSORS_REACTION)  {
                for (i=0; i<N_FORCE_SENSORS; ++i)
                    sensors.reaction[foot][i]=can_message[CAN_DATA1+N_SERVOS+i];
            }
            // Se corresponder aos sensores especiais (inclinmetro e giroscpio)...
            else  {
                sensors.inclinometer[0]=can_message[CAN_DATA1+N_SERVOS+0];
                sensors.inclinometer[1]=can_message[CAN_DATA1+N_SERVOS+1];
                sensors.giroscope[0]=can_message[CAN_DATA1+N_SERVOS+2];
                sensors.giroscope[1]=can_message[CAN_DATA1+N_SERVOS+3];
            }
        }
    }
}

// ****************************************************************************
// Registo de um erro ocorrido com o CAN
//
static void setMcuError(enum CAN_ERRORS error)
{
    byte buffer[4];

    // Erros ocorridos
    canErrors.flags|=error;
    
    // Actualizao dos contadores de erros
    if (canErrors.tx_count<0xFF && (error & TX_BITS))  canErrors.tx_count++;
    if (canErrors.rx_count<0xFF && (error & RX_BITS))  canErrors.rx_count++;

    // Preenchimento da mensagem a enviar pela USART
    buffer[0]=canErrors.flags;
    buffer[1]=canErrors.tx_count;
    buffer[2]=canErrors.rx_count;
    buffer[3]=HOST_MCU;

    // Armazenamento da mensagem num buffer externo
    // S quando for pedido, ela circular para o PC
    usartStore2extBuff(buffer,4);
}

// ****************************************************************************
// Transferncia de erros de um SCU para o MCU
//
static void setScuError(byte scu_id, byte *can_message)
{
    byte buffer[4];

    // Preenchimento da mensagem a enviar pela USART
    buffer[0]=can_message[0];       // Flags indicativas das causas
    buffer[1]=can_message[1];       // Nmero de erros de transmisso
    buffer[2]=can_message[2];       // Nmero de erros de recepo
    buffer[3]=scu_id;               // SCU fonte causador dos erros

    // Armazenamento da mensagem num buffer externo
    // S quando for pedido, ela circular para o PC
    usartStore2extBuff(buffer,4);
}
