/*-----------------------------------------------------------------------+
|             DEPARTAMENTO MECANICA UNIVERSIDADE AVEIRO                  |
|                                                                        |
| Nome:                                                                  |
|       David Manuel Costa Gameiro          N:20281                      |
|       Filipe Carvalho Mostardinha         N:22085                      |
|                                                                        |
| Data:   10/Maio/2004     Nome Ficheiro: InitCan.c                      |
+-----------------------------------------------------------------------*/

#include "can_msg.h"
#include "Can18xx8.h"

// Comprimento das mensagens CAN (bytes)
#define CAN_MESSAGE_LENGTH   8

// Alguns bytes das mensagens CAN (de recepo)
#define CAN_TYPE    0
#define CAN_DATA1   1

// _____ IDENTIFICADOR DAS MENSAGENS CAN ______________________________________________________
// Constantes de referncia ao identificador das mensagens CAN
enum MESSAGE_IDENTIFIER  {
    OPERATION_MASK = 0b11000000000,
    XCUSRC_MASK    = 0b00011110000,
    XCUDST_MASK    = 0b00000001111,

    OPERATION_LOC  = 9,
    XCUSRC_LOC     = 4,
    XCUDST_LOC     = 0,

    OP_UPDATE_SENSORS   = 0b00,
    OP_UPDATE_ACTUATORS = 0b01,
    OP_SLAVECAN_STATUS  = 0b10
};

// Extrao de valores do identificador
#define operation(x) (((unsigned int) (x) & OPERATION_MASK) >> OPERATION_LOC)
#define xcuSRC(x)    (((unsigned int) (x) & XCUSRC_MASK   ) >> XCUSRC_LOC   )
#define xcuDST(x)    (((unsigned int) (x) & XCUDST_MASK   ) >> XCUDST_LOC   )

// Construo de padres do identificador
#define operation_patt(y) (((unsigned int) (y) << OPERATION_LOC) & OPERATION_MASK)
#define xcuSRC_patt(y)    (((unsigned int) (y) << XCUSRC_LOC   ) & XCUSRC_MASK   )
#define xcuDST_patt(y)    (((unsigned int) (y) << XCUDST_LOC   ) & XCUDST_MASK   )

// Construo do identificador
#define identifierMSG(op) (operation_patt(op) | xcuSRC_patt(HOST_SCU) | xcuDST_patt(HOST_MCU))
// ___________________________________________________________________________________________

// PACOTE DE TIPO DAS MENSAGENS OP_UPDATE_SENSORS ____________________________________________
// Constantes de referncia ao byte de tipo dos pacotes de recepo
enum MESSAGE_UPDATE_SENSORS  {
    // Mscaras para extrao de um campo
    SCUSRC_MASK   = 0b11110000,
    MSGINDEX_MASK = 0b00001000,
    SPECIAL_MASK  = 0b00000100,
    SENSORS_MASK  = 0b00000010,
    FOOTID_MASK   = 0b00000001,

    // Localizao dos campos no byte de tipo
    SCUSRC_LOC    = 4,
    MSGINDEX_LOC  = 3,
    SPECIAL_LOC   = 2,
    SENSORS_LOC   = 1,
    FOOTID_LOC    = 0,

    // Identificao do pacote
    MSGINDEX_1  = 0,
    MSGINDEX_2 = 1
};

// Macros para leitura dos diversos campos no byte de tipo
#define scuSRC_patt(y)   (byte) (((unsigned int) (y) << SCUSRC_LOC  ) & SCUSRC_MASK  )
#define msgIndex_patt(y) (byte) (((unsigned int) (y) << MSGINDEX_LOC) & MSGINDEX_MASK)
#define special_patt(y)  (byte) (((unsigned int) (y) << SPECIAL_LOC ) & SPECIAL_MASK )
#define sensors_patt(y)  (byte) (((unsigned int) (y) << SENSORS_LOC ) & SENSORS_MASK )
#define footID_patt(y)   (byte) (((unsigned int) (y) << FOOTID_LOC  ) & FOOTID_MASK  )

// Macros para construo do byte de tipo
#define typePacket(pack_id,sp,sens,foot) (scuSRC_patt(HOST_SCU) | msgIndex_patt(pack_id) | special_patt(sp) | sensors_patt(sens) | footID_patt(foot))
// ___________________________________________________________________________________________________________

// _____ PACOTE DE ESTADO DO SISTEMA _________________________________________________________________________
// Pacote de estado do sistema (ltimo byte das mensagens OP_READ_JOINT
enum MESSAGE_SYSSTATUS_PACKET {
    // Mscaras dos bits
    DEADLINEERROR_MASK = 0b00000100,     // Ocorrncia de violao de deadline
    CONTROLON_MASK     = 0b00000010,     // Controlador PID ligado
    RESET_MASK         = 0b00000001,     // Motores nas home positions

    // Localizao dos bits
    DEADLINEERROR_LOC  = 2,
    CONTROLON_LOC      = 1,
    RESET_LOC          = 0
};

// Construes dos padres do pacote de estado do sistema
#define deadlineError_patt(d) (byte) (((unsigned int) (d) << DEADLINEERROR_LOC) & DEADLINEERROR_MASK)
#define controlOn_patt(c)     (byte) (((unsigned int) (c) << CONTROLON_LOC    ) & CONTROLON_MASK    )
#define reset_patt(r)         (byte) (((unsigned int) (r) << RESET_LOC        ) & RESET_MASK        )

// Construo do pacote de estado do sistema
#define sysStatusPacket(res,cOn,d)   (reset_patt(res) | controlOn_patt(cOn) | deadlineError_patt(d))
// _________________________________________________________________________________________________________

// _____ PACOTE DE TIPO DAS MENSAGENS OP_SLAVECAN_STATUS ___________________________________________________
// Estados de erro relativos ao CAN
enum CAN_ERRORS  {
    TX_FAILED           = 0b10000000,       // Transmisso falhada
    RX_OVERFLOW_ERROR   = 0b00000001,       // Erro de overflow dos buffers de recepo
    RX_INVALIDMSG_ERROR = 0b00000010,       // Mensagem invlida
    RX_INVALID_LENGTH   = 0b00000100,       // Comprimento invlido
    RX_INVALID_DATA     = 0b00001000,       // Dados da mensagem invlidos
    RX_INVALID_OPERATION= 0b00010000,       // Operao invlida
    RX_UNKNOWN_ERROR    = 0b00100000,       // Erro desconhecido

    TX_BITS = 0b10000000,                   // Bits relativos  transmisso
    RX_BITS = 0b01111111                    // Bits relativos  recepo
};
// __________________________________________________________________________________________________________

// Estrutura descritiva dos erros ocorridos
struct  {
    byte flags;                     // Causas
    byte tx_count, rx_count;        // Nmero de ocorrncias
} canErrors={false,0,0};


static bool sendCanStatus(void);                    // Transmisso de uma mensagem de erro
static void updateSensors(byte *msg1,byte *msg2);
static void updateActuators(byte *can_message);     // Actualizao dos dados sensoriais do sistema
static void setError(enum CAN_ERRORS error);        // Registo de um erro ocorrido com o CAN


// ****************************************************************************
//Macros relativas ao CAN, todos os valores sao obtidos tendo em conta um
//um baud rate de 1 Mbps
#define FASE_SYNC 2	    //	Time Quantums para a fase de sincronismo
#define FASE_PROP 2		//	Time Quantums para a fase de propagao
#define FASE_SEG1 2		//	Time Quantums para o segmento 1
#define FASE_SEG2 2		//	Time Quantums para o segmento 2
#define BRP       3 	//  Baud Rate

// Mascara autorizando a que o filtro verifique apenas os 4 bits menos
// significativos (SCU ID)
#define MASCARA XCUDST_MASK

//o byte ob1abcdefg , definido da seguinte forma
// a,b - Receive Buffer Mode bits
// c - Receive Buffer 0 Double-buffer enable bit
// d - qual o identificador a usar (1) standard (0) expandHostHostIDo
// e - amostrar 3 vezes o Bus antes do ponto amostra
// f - acordar o CAN atrav,s do bus, (1) Sim  (0) Nao
// g - Segmento 2 baud rate programado livremente ou nao
#define CONFIGURACAO 0b10101001 //O tipo de configuracao desejada

// Inicializao do CAN
//
void initCan(void)
{
    // Programacao dos pinos RB3 e RB2 como entrada e sada respectivamente
	TRISB=(TRISB | 0x08) & 0xFB;

	// Colocar o MCU em modo configuracao relativamente ao CAN
	CANSetOperationMode(CAN_OP_MODE_CONFIG);

    // Inicializao do Baud Rate do CAN
	CANInitialize(FASE_SYNC, BRP, FASE_SEG1,
				  FASE_SEG2, FASE_PROP, CONFIGURACAO);

	//Definicao das Mascaras e Filtros
	CANSetMask(CAN_MASK_B1, MASCARA, CAN_CONFIG_STD_MSG);
	CANSetMask(CAN_MASK_B2, MASCARA, CAN_CONFIG_STD_MSG);

	CANSetFilter(CAN_FILTER_B1_F1, HOST_SCU, CAN_CONFIG_STD_MSG);
	CANSetFilter(CAN_FILTER_B1_F2, HOST_SCU, CAN_CONFIG_STD_MSG);

	CANSetFilter(CAN_FILTER_B2_F1, HOST_SCU, CAN_CONFIG_STD_MSG);
	CANSetFilter(CAN_FILTER_B2_F2, HOST_SCU, CAN_CONFIG_STD_MSG);
	CANSetFilter(CAN_FILTER_B2_F3, HOST_SCU, CAN_CONFIG_STD_MSG);
	CANSetFilter(CAN_FILTER_B2_F4, HOST_SCU, CAN_CONFIG_STD_MSG);

	//Activar o modo Normal de Funcionamento
	CANSetOperationMode(CAN_OP_MODE_NORMAL);

    // Activa os interrupts dos buffers recepcao CAN
    IPR3bits.RXB0IP=0;                                  // Low priority interrupts
    IPR3bits.RXB1IP=0;
    PIE3bits.RXB0IE=1;                                  // Interrupts Enable
    PIE3bits.RXB1IE=1;
}

// ****************************************************************************
/////////////////////////////////////////////////////////////////////////
// Macro CAN, configuracao envio mensagem                              //
//																	   //
// byte definido 0b1a111bcd											   //
// a - pretende-se notificacao de recepcao de msg (1) caso contrario 0 //
// b - indica qual o tipo de indentificador (1)STD (0)XTD			   //
// [c,d] - prioridade a dar a esta mensagem, ver registo TXBnCON	   //
/////////////////////////////////////////////////////////////////////////

#define CAN_TX_FLAGS 0b11000111

// Transmisso de uma mensagem CAN relativa aos actuadores do sistema
//
bool sendCanMessage(void)
{
    bool res1, res2, res3;
    unsigned int identifier;
    byte message1[CAN_MESSAGE_LENGTH], message2[CAN_MESSAGE_LENGTH];

    // Construo das duas mensagens com o contedo sensorial
    updateSensors(message1,message2);

    // Identificador a utilizar nas mensagens a enviar
    identifier=identifierMSG(OP_UPDATE_SENSORS);

    //funao que envia mensagens
    if (!(res1=CANSendMessage(identifier, message1, CAN_MESSAGE_LENGTH, CAN_TX_FLAGS)))
        setError(TX_FAILED);
    if (!(res2=CANSendMessage(identifier, message2, CAN_MESSAGE_LENGTH, CAN_TX_FLAGS)))
        setError(TX_FAILED);

    // Caso tenha ocorrido algum erro no passado, transmitir
    // mensagem extra com informaes detalhadas
    if (canErrors.flags && (res3=sendCanStatus()))  canErrors.flags=0;

    // Apenas retorna true, se as duas transmisses tiveram sucesso!
    return (res1 && res2 && res3);
}


// ****************************************************************************
// Recepo de uma mensagem CAN de um dos SCU
//
bool receiveCanMessage(void)
{
    unsigned long id;
    byte length;
    enum CAN_RX_MSG_FLAGS warnings;
    byte message[CAN_MESSAGE_LENGTH];

    // Um erro ocorreu na recepo
    if (!CANReceiveMessage(&id, message, &length, &warnings))
        return false;

    // Erro de Overflow
    if (warnings & CAN_RX_OVERFLOW)  {
//        setError(RX_OVERFLOW_ERROR);
  		COMSTAT &= 0b00111111;      //retirar o overflow
    }
    // Erro de mensagem invlida
    else if (warnings & CAN_RX_INVALID_MSG)  {
//        setError(RX_INVALIDMSG_ERROR);
  		PIR3bits.IRXIF = 0;
    }

    // Comprimento de dados invlido
    if (length<CAN_MESSAGE_LENGTH)
        setError(RX_INVALID_LENGTH);

    // Tudo OK!
    else if (operation(id)==OP_UPDATE_ACTUATORS)
        updateActuators(message);         // Actualizao dos actuadores

    else
        setError(RX_UNKNOWN_ERROR);  // Erro: operao no vlida!

    if (canErrors.flags)  PORTCbits.RC0=1;
    else    PORTCbits.RC0=0;

    return true;
}


// ****************************************************************************
// Actualizao dos dados sensoriais do sistema
//
static void updateActuators(byte *can_message)
{
    byte i;

    // Actualizar as posies e velocidades dos servos da junta
    for (i=0; i<N_SERVOS; ++i)  {
        actuators.servo[i].position=can_message[i];
        actuators.servo[i].velocity=can_message[i+N_SERVOS];
    }
    actuators.control.kp=can_message[2*N_SERVOS+0];
    actuators.control.kd=can_message[2*N_SERVOS+1];

    // REMOVER
    for (i=0; i<N_SERVOS; ++i)  {
        sensors.servo[i].position=actuators.servo[i].position;
        sensors.servo[i].velocity=actuators.servo[i].velocity;
    }
    sensors.sysStatus.controlOn = (actuators.control.kp==0 && actuators.control.kd==0) ? false : true;

    for (i=0; i<N_SERVOS && actuators.servo[i].position==-128; ++i);
    sensors.sysStatus.reset = (i==N_SERVOS) ? true : false;
    // REMOVER
}

static void updateSensors(byte *msg1, byte *msg2)
{
    byte i, ind;

    // Preenchimento do primeiro pacote
    // Byte de tipo
    ind=0;
    msg1[ind++]=typePacket(MSGINDEX_1,0,0,0);
    for (i=0; i<N_SERVOS; ++i,++ind)  {
        msg1[ind]=sensors.servo[i].position;
        msg1[N_SERVOS+ind]=sensors.servo[i].velocity;
    }
    ind+=N_SERVOS;      // Actualizao do ndice da mensagem CAN tendo em conta as velocidades introduzidas
    // Pacote de estado do sistema
    msg1[ind++]=sysStatusPacket(sensors.sysStatus.reset,sensors.sysStatus.controlOn,sensors.sysStatus.deadlineError);
    // Finalizao do preenchimento com zeros
    while (ind<CAN_MESSAGE_LENGTH)  msg1[ind++]=0;

    // Preenchimento do segundo pacote
    // Byte de tipo
    ind=0;
    msg2[ind++]=typePacket(MSGINDEX_2,SPECIAL_SENSORS,SENSORS_ID,FOOT_ID);
    // Dados das correntes consumidas pelos servos
    for (i=0; i<N_SERVOS; ++i,++ind)
        msg2[ind]=sensors.servo[i].current;
    // Se h sensores especiais
    if (SPECIAL_SENSORS)  {
        // No caso de sensores de reaco...
        if (SENSORS_ID==SENSORS_REACTION)
            for (i=0; i<N_FORCE_SENSORS; ++i,++ind)
                msg2[ind]=sensors.reaction[i];
        else  {
            // Inclinmetro/Giroscpio
            msg2[ind++]=sensors.inclinometer[0];
            msg2[ind++]=sensors.inclinometer[1];
            msg2[ind++]=sensors.giroscope[0];
            msg2[ind++]=sensors.giroscope[1];
        }
    }
    // Finalizao do preenchimento com zeros
    while (ind<CAN_MESSAGE_LENGTH)  msg2[ind++]=0;
}

static bool sendCanStatus(void)
{
    bool res;
    byte i, message[CAN_MESSAGE_LENGTH];
    unsigned int identifier;

    // Identificador da mensagem
    identifier=identifierMSG(OP_SLAVECAN_STATUS);

    // Preenchimento de um array com a informao dos erros ocorridos para transmisso
    message[0]=canErrors.flags;         // Flags com a identificao dos erros ocorridos
    message[1]=canErrors.tx_count;      // Nmero de erros de transmisso
    message[2]=canErrors.rx_count;      // Nmero de erros de recepo
    message[3]=HOST_SCU;                // Identificao do SCU fonte

    // Finalizao do preenchimento com zeros
    for (i=4; i<CAN_MESSAGE_LENGTH; ++i)  message[i]=0;

    // Transmisso da mensagem
    if (!(res=CANSendMessage(identifier, message, CAN_MESSAGE_LENGTH, CAN_TX_FLAGS)))
        // Caso ocorra um erro, assinal-lo
        setError(TX_FAILED);

    return res;
}

// ****************************************************************************
// Registo de um erro ocorrido com o CAN
//
static void setError(enum CAN_ERRORS error)
{
    // Erros ocorridos
    canErrors.flags|=error;
    
    // Actualizao dos contadores de erros
    if (canErrors.tx_count<0xFF && (error & TX_BITS))  canErrors.tx_count++;
    if (canErrors.rx_count<0xFF && (error & RX_BITS))  canErrors.rx_count++;
}
