/*=======================================================================+
|             DEPARTAMENTO MECANICA UNIVERSIDADE AVEIRO                  |
|               Projecto Humanoide (PHUA - MODULROB)                     |
|                                                                        |
| Programadores:                                                         |
|              David Manuel Costa Gameiro          N:20281               |
|              Filipe Carvalho Mostardinha         N:22085               |
|                                                                        |
| Nome Ficheiro: InitPic.c                                               |
|                                                                        |
|                            SOURCE FILE                                 |
+=======================================================================*/


#include "pic.h"
#include "usart.h"
#include "can.h"

void isr(void);                             // Rotina de servio a interrupes de alta prioridade


// Quando ocorre um interrupt, o programa "salta" para esta linha do programa,
// que consequentemente reencaminha para a rotina responsvel pelo tratamento das interrupes.
#pragma code isrVector = 0x08
void isrVector(void)
{
_asm
	goto isr			                    // Salta para rotina de interrupt prioritario
_endasm
}
#pragma code

void initPic(void)
{
    unsigned long i;


    /*------------------------------------------------------------------+
    |                    Definicoes dos TRIS do PIC                     |
    +------------------------------------------------------------------*/
    TRISA = 0;
    TRISB = 0;
    TRISC = 0;

    /*------------------------------------------------------------------+
    |                    Configurao do porto RA                       |
    +------------------------------------------------------------------*/

	ADCON1=0b00000110;      // All RA pins are digital

    /*------------------------------------------------------------------+
    |          			 Inicializao das Comunicaes       		    |
    +------------------------------------------------------------------*/

    usartInit();            // Inicializao da USART
    canInit();              // Inicializao do CAN

    // Espera de 100 ms para garantir que todos os SCU esto
    // em plena operao!
    wait(1000);

    /*------------------------------------------------------------------+
    |          			 Definies dos Interrupts             		    |
    +------------------------------------------------------------------*/

    // Activacao global das interrupcoes
	INTCONbits.PEIE=1;		// Enable all peripheral interrupt sources
	INTCONbits.GIE =1;      // Enable all interrupt sources
}

// Bloqueio do PIC em <val> ciclos de 100us (para fosc=40MHz)
//
void wait(unsigned int val)
{
    unsigned int i, j;

    for (i=0; i<val; ++i)
        for (j=0; j<140; ++j);
}


// Rotina responsvel pelo tratamento das interrupes de alta prioridade
//
#pragma interrupt isr
void isr(void)
{
    // Byte recebido pela USART
    byte rx;
    // Indicadores do estado dos buffer da USART
    bool full, empty;
    // Mscaras para identificao da origem do agendamento de transmisso pela USART
    static enum  {
        ERROR_MASK      = 0b00000001,
        SENDLAST_MASK   = 0b00000010,
        SENDSTREAM_MASK = 0b00000100,
        SENDTEST_MASK   = 0b00001000,
        CANCELTX_MASK   =(SENDLAST_MASK | SENDTEST_MASK)
    } txAgenda=false;
    // SCU ID a enviar a mensagem CAN
    static byte scu2send=HOST_MCU+1;

    //_____ COMUNICAO CAN ____________________________________________________________
    // Recepo de uma mensagem via CAN, de um dos SCU's
    if (PIR3bits.RXB1IF || PIR3bits.RXB0IF)  canReceiveMsg();

    //_____ COMUNICAO PELA USART ____________________________________________________

    // Sempre que uma entrada  feita na RSI, as interrupes
    // do Timer de retransmisso (Timer0) devem ser desactivadas!
    INTCONbits.TMR0IE=0;

    // Interrupo causada pela receo de um byte pela USART
    if (PIR1bits.RCIF)  {
        // Apenas processa o byte recebido, se no houver nenhuma transmisso agendada,
        if (!txAgenda)  {
            // Verificao da ocorrncia de um Framing Error
            // Este erro  provocado por malformaes da palavra recebida
            if (RCSTAbits.FERR)  {
                RCREG;                                  // Error cleared
                txAgenda|=ERROR_MASK;                   // Agendamento da transmisso de uma mensagem de erro
            }
            // Overrun Error
            // Este erro ocorre quando um caracter  recebido com o buffer
            // de recepo (FIFO) cheio!
            else if (RCSTAbits.OERR)  {
                RCREG;                                  // Remoo de um byte do buffer de recepo da USART
                RCSTAbits.CREN = 0;                     // Error cleared
                RCSTAbits.CREN = 1;
       	    	txAgenda|=ERROR_MASK;                   // Agendamento da transmisso de uma mensagem de erro
   	        }
            // Tudo OK
            else  {
                // Leitura do byte recebido
                rx = RCREG;

                // Em modo de recepo de um stream de bytes...
                if (usartStreamMode() || rx==MESSAGE_REQ)  {
                    // Armazenamento do byte recebido no buffer de recepo
                    usartStoreRx(rx,&full);

                    // Caso o buffer esteja completo, agendar transmisso do stream
                    if (full)  txAgenda|=SENDSTREAM_MASK;
                }
                // Em caso de recepo de um pacote de erro (fora da recepo de streams)
                // agendar a transmisso da ltima mensagem transmitida
                else if (rx==MESSAGE_ERROR)  txAgenda|=SENDLAST_MASK;
                // Em caso de recepo de um pacote de calibrao, enviar trama sequencial
                else if (rx==MESSAGE_TEST)  txAgenda|=SENDTEST_MASK;
            }
        // Caso j haja uma transmisso agendada, descartar byte recebido!
        } else  RCREG;
    }

    // Transmisso de um stream de bytes
    // Aqui no  considerado o semforo de acesso ao buffer de transmisso, dado que a RSI
    // tem prioridade no seu acesso. A funo que quiser modificar este buffer deve tambm
    // verificar se existe alguma transmisso em curso (TXIE)
    if (PIE1bits.TXIE && PIR1bits.TXIF)  {

        // Se a transmisso no foi cancelada, transmitir o prximo byte do buffer de tx
        if (!(txAgenda & CANCELTX_MASK))
            usartGetTx((byte *) &TXREG,&empty);

        // Verificao de trmino de transmisso (fim de tx do buffer ou cancelamento da transmisso)
        if (empty || (txAgenda & CANCELTX_MASK))  {
            usartResetTxBuff();                     // Reset do buffer de transmisso
            PIE1bits.TXIE=0;                        // Desactivao do envio por interrupo
        }
    }

    // Atendimento de transmisses agendadas. Apenas o faz quando...
    // ... existe uma transmisso agendada,
    if (txAgenda)  {
        // ... o semforo de bloqueio no acesso ao buffer de transmisso est desimpedido,
        if (usartBufftxSemaph())  {
            // ... e no existe nenhuma transmisso em curso.
            if (!PIE1bits.TXIE)  {

                // Transmisso de um stream de bytes
                if (txAgenda & SENDSTREAM_MASK)  usartProcessMsg();

                // Envio de uma mesagem para calibrao das comunicaes
                else if (txAgenda & SENDTEST_MASK)  usartSendTestMsg();

                // Transmisso de uma mensagem de Erro
                else if (txAgenda & ERROR_MASK)     usartSendStatus(MESSAGE_ERROR);

                // No caso de reenvio da ltima mensagem, 
                // suficiente reactivar a transmisso (feito abaixo)

                // Preparao para a recepo de streams
                usartResetRxBuff();                        // Reset do buffer de recepo

                // Clear de agendamento de transmisso
                txAgenda=false;

                // Activao da transmisso
                PIE1bits.TXIE=1;
            }
        }
        // Em caso de semforo bloqueado, voltar a tentar daqui a 100us
        else  {
            TMR0L=kt0;              // Inicializao do contador
            INTCONbits.TMR0IE=1;    // Reactivao das interrupes
            INTCONbits.TMR0IF=0;
        }
    }
    //___________________________________________________________________________________________
}
