/*=======================================================================+
|             DEPARTAMENTO MECANICA UNIVERSIDADE AVEIRO                  |
|               Projecto Humanoide (PHUA - MODULROB)                     |
|                                                                        |
| Programadores:                                                         |
|              David Manuel Costa Gameiro          N:20281               |
|              Filipe Carvalho Mostardinha         N:22085               |
|                                                                        |
| Nome Ficheiro: Tipos.h                                                 |
|                                                                        |
|                            HEADER FILE                                 |
+=======================================================================*/


#ifndef _GLOBAL_H
#define _GLOBAL_H

#include <p18f258.h>

#define HOST_MCU    0   // ID deste MCU

// Frequncia de funcionamento do CPU (MHz) - 1/4 Fosc
#define F_CPU	10

typedef unsigned char byte;
typedef unsigned int  word;

// Definio do tipo booleano
typedef unsigned char bool;

#ifndef true
#define true  1
#endif

#ifndef false
#define false 0
#endif

// Dados sobre o robot
#define N_SCU           1       // Nmero de Slave Controller Units
#define N_SERVOS        3       // Nmero de servos por SCU
#define N_FEET          2       // Nmero de ps
#define N_FORCE_SENSORS 4       // Nmero de sensores de fora por p

// Estrutura descritiva de um servo
typedef struct {
    signed char position;       // Posio
    signed char velocity;       // Velocidade
    unsigned char current;      // Corrente consumida
} struct_servo;

// Estrutura descritiva dos sensores
typedef struct {
    struct {
        bool reset;                         // Motores nas home positions
        bool deadlineError;                 // Violao de deadline
        bool motionFin[N_SERVOS];           // Movimento terminado
        bool controlOn[N_SERVOS];           // Estado on/off dos controladores
    } sysStatus[N_SCU];                                 // Estado do sistema
    struct_servo servo[N_SCU][N_SERVOS];                // Sensores dos servos
    unsigned char reaction[N_FEET][N_FORCE_SENSORS];    // Sensores de fora dos ps
    signed char giroscope[2];                           // Giroscpio
    signed char inclinometer[2];                        // Inclinmetro
} struct_sensors;

typedef struct {
    struct {
        byte kp, kd, ki;
        bool controlOn;
    } control[N_SCU][N_SERVOS];
    struct_servo servo[N_SCU][N_SERVOS];
} struct_actuators;

// Variveis globais descritivas do sistema
extern struct_sensors   sensors;                        // Sensores
extern struct_actuators actuators;                      // Actuadores

// Clculo do ndice dos arrays dos sensores/actuadores correspondentes ao ID do SCU
#define scuIndex(scu_id)    (scu_id-HOST_MCU-1)

extern void initGlobal(void);

extern void initSensors(void);

extern void initActuators(void);

#endif
