/*=======================================================================+
|             DEPARTAMENTO MECANICA UNIVERSIDADE AVEIRO                  |
|               Projecto Humanoide (PHUA - MODULROB)                     |
|                                                                        |
| Programadores:                                                         |
|              David Manuel Costa Gameiro          N:20281               |
|              Filipe Carvalho Mostardinha         N:22085               |
|                                                                        |
| Nome Ficheiro: InitPic.c                                               |
|                                                                        |
|                            SOURCE FILE                                 |
+=======================================================================*/


#include "pic.h"
#include "can.h"


void lowPriorityISR(void);

// Quando ocorre um interrupt de baixa prioridade o programa "salta" para esta 						//linha do programa, cabe a linha de codigo goto CanTimerIsr reencaminhar para a
// rotina que se deseja executar como rotina de baixa prioridade.
#pragma code LowIsrVector = 0x018
void LowIsrVector(void)
{
_asm
	goto lowPriorityISR		        //salta para rotina de interrupt nao prioritario (baixa prio.)
_endasm
}
#pragma code


void initPic(void)
{
    /*------------------------------------------------------------------+
    |                    Definicoes dos TRIS do PIC                     |
    +------------------------------------------------------------------*/
    TRISA = 0;
    TRISB = 0;
    TRISC = 0;
    PORTC=0;

    /*------------------------------------------------------------------+
    |                    Configurao do porto RA                       |
    +------------------------------------------------------------------*/

	ADCON1=0b00000110;      // All RA pins are digital

    /*------------------------------------------------------------------+
    |          			    Definicoes da USART             		    |
    +------------------------------------------------------------------*/

    initCan();

    /*------------------------------------------------------------------+
    |          			 Definies dos Interrupts             		    |
    +------------------------------------------------------------------*/

    // Activacao global das interrupcoes
	RCONbits.IPEN=1;		// Interrupt priority enable
	INTCONbits.GIEH=1;		// Enable high priority interrupts
	INTCONbits.GIEL=1;      // Enable low priority interrupts
}


// Mquina de estados na troca de mensagens CAN
#define N_OPERATIONS    4
typedef enum {
    RECV_MESSAGE1 = 0,
    RECV_MESSAGE2 = 1,
    SEND_MESSAGE1 = 2,
    SEND_MESSAGE2 = 3,
    SEND_MESSAGE3 = 4
} enum_CanExchange_States;


// ****************************************************************************
// RSI de baixa prioridade
//
#pragma interrupt lowPriorityISR
void lowPriorityISR(void)
{
    bool res=true;
    static enum_CanExchange_States can_op=RECV_MESSAGE1;

    //_____ COMUNICAO CAN ____________________________________________________________
    // Envio de uma mensagem, via CAN, para um SCU para actualizao dos seus actuadores
    if (PIR1bits.TMR2IF)  {

        PORTCbits.RC1=1;

        switch (can_op)  {
        // Recepo das mensagens de actualizao dos actuadores
        case RECV_MESSAGE1: res=receiveCanMessage(0);
                            break;
        case RECV_MESSAGE2: res=receiveCanMessage(1);
                            break;
        // Envio da primeira mensagem de actualizao dos actuadores
        case SEND_MESSAGE1: res=sendCanMessage(0);
                            break;
        // Envio da segunda mensagem de actualizao dos actuadores
        case SEND_MESSAGE2: res=sendCanMessage(1);
                            break;
        }

        PORTCbits.RC2=res;

        // Apenas passa para a operao seguinte, se a actual (res) teve sucesso;
        // e apenas muda o slave a operar, quando a mquina de estados volta ao incio.
        if (res && (can_op=(can_op+1)%4)==0)
            PORTCbits.RC3=!PORTCbits.RC3;

        PORTCbits.RC1=0;
        PIR1bits.TMR2IF=0;
    }
    //____________________________________________________________________________________
}
