#ifndef _SERVO_H
#define _SERVO_H

#include "global.h"

// Limiar de deciso para deteco do impulso de corrente acima
// do valor mnimo:
#define VPOT_IMPULSE_LIMIAR	60		// 60/1023*5.0 = 0.29V < 0.5V

#define DUTY_MIN	880  //(((unsigned int) 256-KT0_MAX)*T0_PRES/F_CPU)
#define DUTY_MAX	1900 //(((unsigned int) 256-KT0_MIN)*T0_PRES/F_CPU)

// Propriedades de funcionamento do servo
#define SERVO_RANGE	90              // Gama de funcionamento do servo para cada lado (graus)
#define SERVO_RES   90              // Quantidade de posies possveis: para 90 -> Resoluo=2 (2*RANGE/RES)

// Pinos de interface do PIC
#define SERVO1_PIN	PORTBbits.RB4   // Saida de PWM para o servomotor (perodo de 20ms)
#define SERVO2_PIN	PORTBbits.RB5   // Saida de PWM para o servomotor
#define SERVO3_PIN	PORTBbits.RB6   // Saida de PWM para o servomotor

// Estrutura com os dados sensoriais pr-processodos
typedef struct {
    word vpotMin;               // Tenso mnima num ciclo de PWM
    byte vpotImp;               // Durao do impulso de tenso num ciclo de PWM
} struct_adcAnalog;
extern struct_adcAnalog adcServo[N_SERVOS];

// Estrutura com os dados a aplicar nos actuadores
typedef struct  {
    byte pwm;                   // PWM associado ao ngulo solicitado
    signed int angle;           // ngulo a solicitar
    signed int error_prev;      // Erro do ciclo anterior entre a posio efectiva e a desejada
} struct_pwmServo;
extern struct_pwmServo pwmServo[N_SERVOS];

#define ANGLE_FACTOR    100

// Dados do controlador
// extern float controlRequested;

extern void initServoData(void);

extern void transferServoSensors(void);

extern void controller(void);

#endif
