#include "servo.h"

const signed int origin[N_SERVOS]=ORIGIN;

// Macro conversora de ngulo para unidades de PWM
#define PWM(ang)    (byte) ((SERVO_RANGE-(signed int) (ang))*SERVO_RES/(2*(signed int) SERVO_RANGE))

// Estruturas externas
volatile struct_pwmServo pwmServo[N_SERVOS];     // Dados dos actuadores
volatile struct_velocity velocityServo[N_SERVOS]; // Dados dos sensores


// Inicializao das estruturas de dados
//
void initServoData(void)
{
    byte srv, i;

    for (srv=0; srv<N_SERVOS; ++srv)  {

        // Inicialmente todos os pwm esto desligados
        pwmServo[srv].pwmOn=false;

        // Inicializao das posies dos servos nos valores standard
        pwmServo[srv].applyPosition=homePosition[srv];
        pwmServo[srv].pwm=PWM(homePosition[srv]);

        // Inicializao dos arrays de posio
        for (i=0; i<N_POS; ++i)  {
            velocityServo[srv].positions[i]=homePosition[srv];
            velocityServo[srv].oldPosition=0;
        }
    }
}

// Inicializao do Controlador
//
void initController(void)
{
    byte srv;

    for (srv=0; srv<N_SERVOS; ++srv)  {

        pwmServo[srv].control.angle=(double) sensors.servo[srv].position;
        pwmServo[srv].control.deltaD_prev=0.0;
        pwmServo[srv].control.error_prev=0;
        pwmServo[srv].control.y_prev=0;
        pwmServo[srv].control.y_prev2=0;
    }
}

// Aplicao de uma nova velocidade a um servo
//
void servoPos(signed char position, byte srv)
{
    if (position<-SERVO_RANGE)       position=-SERVO_RANGE;
    else if (position>SERVO_RANGE)   position= SERVO_RANGE;

    pwmServo[srv].applyPosition=position;
}


// Funcao para aquisicao do sinal presente no potencimetro do servo
//
/*
// Os seguintes valores so utilizados para converter uma tenso entre 0 e 5000 num ngulo em graus.
// POS(graus)=ORIGIN-<tenso>*SLOPE/QUOCIENT
#define SLOPE      9        // Declive=1800     => Simplificando 1800/10000=9/50: SLOPE=9
#define QUOCIENT   50       // Quociente=10000  => Simplificando 1800/10000=9/50: QUOCIENT=50
#define ORIGIN     236      // Ordenada na origem
#define POS(volt) (ORIGIN - (signed int) ((word) (volt)*SLOPE/QUOCIENT));

// Funo bloqueante de processamento das medidas
//
void transferServoSensors(word vpotMin, byte vpotImp, byte srv)
{
	signed int ang, ang_old;
    struct_velocity *ptr=velocityServo+srv;

    // Clculo da posio em graus
    ang=POS(vpotMin);                   // Clculo do ngulo associado  tenso medida
    if (ang<-127)     ang=-127;         // Truncagem do resultado de modo a caber numa varivel de 8 bits
    else if (ang>127) ang= 127;

    // Leitura da medio de posio mais antiga
    ang_old=ptr->positions[ptr->oldPosition];
    // A nova posio medida  armazenada no elemento mais antigo
    ptr->positions[ptr->oldPosition]=ang;
    // Actualizao do elemento do array de posies com a medio mais antiga
    ptr->oldPosition=(ptr->oldPosition+1)%N_POS;

    //ang=vpotMin;     // Clculo do ngulo associado  tenso medida

    sensors.servo[srv].position=(signed char) ang;      // Posio angular
    sensors.servo[srv].velocity=ang-ang_old;            // Velocidade (graus/N_POS ticks)
    sensors.servo[srv].current =vpotImp;                // Corrente
}
*/
//#define KP  0.10  //0.17        // 0.10 ou 0.07 Para a gama -90 a 0 estes // 0.06 ou 0.06 -45=>+45 //  0.05    Destino: Binrio0
//#define KD  0.45  //0.90        // 0.15    0.00 valores so os ideais     // 0.06    0.00          //  0.00

#define FILTER_N  1.0             // Nvel de filtragem da componente derivativa

void controller(void)
{
    byte srv;
    signed char posReference, y_now;
    signed int error, error_dif, y_vardif;
    double deltaP, deltaI, deltaD, kp, ki, kd;

    // Aplicao do controlador
    for (srv=0; srv<N_SERVOS; ++srv)  {

        posReference=pwmServo[srv].applyPosition;           // Posio desejada (referncia)
        y_now = sensors.servo[srv].position;                // Posio efectiva do servo

        // Erro actual
   	    error=(signed int) posReference - y_now;
        // Introduo da banda morta
        if (error>-2 && error<2)    error= 0;
        else if (error== 3)         error= 2;
        else if (error==-3)         error=-2;

   	    // Diferena do erro para a aco proporcional e variao da diferena da sada para a aco derivativa
        error_dif= error-pwmServo[srv].control.error_prev;
        y_vardif =(signed int) y_now-2*pwmServo[srv].control.y_prev+pwmServo[srv].control.y_prev2;

   	    // Actualizao das sadas/erros anteriores
        pwmServo[srv].control.error_prev=error;
        pwmServo[srv].control.y_prev2=pwmServo[srv].control.y_prev;
        pwmServo[srv].control.y_prev =y_now;

        // No caso do controlador ligado, aplicar compensao
        if ((sensors.sysStatus.controlOn[srv] = actuators.control[srv].controlOn
            && (actuators.control[srv].kp!=0 || actuators.control[srv].kd!=0 || actuators.control[srv].ki!=0)))  {

            ki = (double) actuators.control[srv].ki/(double) ANGLE_FACTOR;
            kp = (double) actuators.control[srv].kp/(double) ANGLE_FACTOR;
            kd = (double) actuators.control[srv].kd/(double) ANGLE_FACTOR;

            // Componentes do compensador
            deltaI = (double) error*ki;                     // Compensao integrativa
            deltaP = (double) error_dif*kp;                 // Compensao proporcional
            deltaD = (double) y_vardif*kd;                  // Compensao derivativa

            // Filtragem
//            deltaD = (pwmServo[srv].control.deltaD_prev*kd/FILTER_N + deltaD)/(1+kd/FILTER_N);
            pwmServo[srv].control.deltaD_prev=deltaD;       // Update da ltima variao de derivada

            // Clculo da nova posio a aplicar
            pwmServo[srv].control.angle += deltaI + deltaP + deltaD;

       	    // Limitao da posio a solicitar
            if ((signed int) pwmServo[srv].control.angle<-SERVO_RANGE)
                pwmServo[srv].control.angle=-(double) SERVO_RANGE;
       	    else if ((signed int) pwmServo[srv].control.angle>SERVO_RANGE)
                pwmServo[srv].control.angle= (double) SERVO_RANGE;
        }
        // Caso o controlador esteja desligado,
        // aplicar imediatamente a posio desejada
        else
            pwmServo[srv].control.angle=(double) posReference;


        // Converso de posio para PWM
        pwmServo[srv].pwm=PWM((signed char) pwmServo[srv].control.angle);

        // Activao da gerao de PWM
        pwmServo[srv].pwmOn=true;
    }
}
