#ifndef _SERVO_H
#define _SERVO_H

#include "global.h"

// Limiar de deciso para deteco do impulso de corrente acima
// do valor mnimo:
#define VPOT_IMPULSE_LIMIAR	50		// 50/1023*5.0 = 0.244V < 0.5V

#define DUTY_MIN	890  //(((unsigned int) 256-KT0_MAX)*T0_PRES/F_CPU)
#define DUTY_MAX	1920 //(((unsigned int) 256-KT0_MIN)*T0_PRES/F_CPU)

// Propriedades de funcionamento do servo
#define SERVO_RANGE	90              // Gama de funcionamento do servo para cada lado (graus)
#define SERVO_RES   180             // Quantidade de posies possveis: para 90 -> Resoluo=2 (2*RANGE/RES)

// Pinos de interface do PIC
#define SERVO1_PIN	PORTBbits.RB4   // Saida de PWM para o servomotor (perodo de 20ms)
#define SERVO2_PIN	PORTBbits.RB5   // Saida de PWM para o servomotor
#define SERVO3_PIN	PORTBbits.RB6   // Saida de PWM para o servomotor


// Estrutura com dados necessrios  medio de velocidade
#define N_POS   5                   // Nmero de medies a armazenar no array de posies
typedef struct {
    signed char positions[N_POS];   // Array com as ltimas medies de posio
    signed char oldPosition;        // ndice do array de posies onde est armazenado a medio mais antiga
} struct_velocity;
extern volatile struct_velocity velocityServo[N_SERVOS];


// Estrutura com os dados a aplicar nos actuadores
typedef struct  {
    signed char applyPosition;      // Nova posio angular a aplicar
    byte pwm;                       // PWM associado  posio solicitada
    struct {
        signed int angle;           // ngulo a solicitar
        signed int error_prev;      // Erro do ciclo anterior entre a posio efectiva e a desejada
    } control;
} struct_pwmServo;
extern volatile struct_pwmServo pwmServo[N_SERVOS];

#define ANGLE_FACTOR    100

// Dados do controlador
// extern float controlRequested;

extern void initServoData(void);

// Aplicao de uma nova velocidade a um servo
extern void servoPos(signed char position, byte srv);

extern void transferServoSensors(word vpotMin, byte vpotImp, byte srv);

extern void controller(void);

#endif
