#ifndef _SERVO_H
#define _SERVO_H

#include "global.h"

// Limiar de deciso para deteco do impulso de corrente acima
// do valor mnimo:
#define VPOT_IMPULSE_LIMIAR	50		// 50/1023*5.0 = 0.244V < 0.5V
#define DUTY_MIN	930  //890  // CUIDADO COM ESTE VALOR: O perodo do Timer2 no deve fazer overflow!
#define DUTY_MAX	1960 //1920 //

// Propriedades de funcionamento do servo
#define SERVO_RANGE	90              // Gama de funcionamento do servo para cada lado (graus)
#define SERVO_RES   180             // Quantidade de posies possveis: para 90 -> Resoluo=2 (2*RANGE/RES)

// Pinos de interface do PIC
#define SERVO1_PIN	PORTBbits.RB4   // Saida de PWM para o servomotor (perodo de 20ms)
#define SERVO2_PIN	PORTBbits.RB5   // Saida de PWM para o servomotor
#define SERVO3_PIN	PORTBbits.RB6   // Saida de PWM para o servomotor


// Estrutura com dados necessrios  medio de velocidade
#define N_POS   10                  // Nmero de medies a armazenar no array de posies
typedef struct {
    signed char positions[N_POS];   // Array com as ltimas medies de posio
    signed char oldPosition;        // ndice do array de posies onde est armazenado a medio mais antiga
} struct_velocity;
extern volatile struct_velocity velocityServo[N_SERVOS];


// Estrutura com os dados a aplicar nos actuadores
typedef struct  {
    signed char applyPosition;      // Nova posio angular a aplicar
    byte pwm;                       // PWM associado  posio solicitada
    struct {
        double angle;               // ngulo a solicitar
        signed int error_prev;      // Erro do ciclo anterior entre a posio efectiva e a desejada
    } control;
} struct_pwmServo;
extern volatile struct_pwmServo pwmServo[N_SERVOS];

#define ANGLE_FACTOR    100

// Os seguintes valores so utilizados para converter uma tenso entre 0 e 5000 num ngulo em graus.
// POS(graus)=ORIGIN-<tenso>*SLOPE/QUOCIENT
#define SLOPE      9        // Declive=1800     => Simplificando 1800/10000=9/50: SLOPE=9
#define QUOCIENT   50       // Quociente=10000  => Simplificando 1800/10000=9/50: QUOCIENT=50
#define ORIGIN     243 //236      // Ordenada na origem
#define POS(volt) (ORIGIN - (signed int) ((word) (volt)*SLOPE/QUOCIENT));

// Dados do controlador
// extern float controlRequested;

extern void initServoData(void);

// Aplicao de uma nova velocidade a um servo
extern void servoPos(signed char position, byte srv);

//extern void transferServoSensors(word vpotMin, byte vpotImp, byte srv);

extern void controller(void);

#endif
