/*********************************************************************
 *              PIC18FXX8 CAN C Library Header File
 *********************************************************************
 * FileName:        CAN18FXX8.h
 * Date:            11/03/2004
 * Processor:       PIC18FXX8
 * Complier:        MPLAB 6.40.00
 *
 * defined for use with MPLAB C18
 *********************************************************************/
 
#ifndef CANDRIVERS_H          // To avoid duplicate inclusion
#define CANDRIVERS_H

#ifndef true
#define true 1
#endif

#ifndef false
#define false 0
#endif

#ifndef NULL
#define NULL    (void *) 0
#endif

typedef unsigned char bool;

typedef unsigned char byte;
typedef unsigned int  word;

// Comprimento mximo dos dados em bytes
#define DATA_LENGTH     8

// ****************************************************************************
//Macros relativas ao CAN, todos os valores sao obtidos tendo em conta um
//um baud rate de 1 Mbps
// Note que a fase de salto NO  a fase de sincronizao. Essa possui
// sempre um Tq=1. A fase de salto apenas apenas se verifica na ressincronizao
// e  menor ou igual a FASE_JUMP Tq's.
// Regras para definir a nmero de Tq para cada fase:
// 1- (1+FASE_PROP+FASE_SEG1+FASE_SEG2)>=8 Tq's
// 2- FASE_PROP+FASE_SEG1>=FASE_SEG2
// 3- FASE_SEG2>=FASE_JUMP
#define FASE_JUMP 2	    //	Time Quantums para a fase de ressincronizao
#define FASE_PROP 2		//	Time Quantums para a fase de propagao
#define FASE_SEG1 3		//	Time Quantums para o segmento 1
#define FASE_SEG2 2		//	Time Quantums para o segmento 2

#define BRP       3 	//  Baud Rate

enum CAN_STATUS_TX
{
    CAN_STATUS_ERROR    = 0b10000000,
    CAN_STATUS_TXERR    = 0b01000000,
    CAN_STATUS_TXLARB   = 0b00100000,
    CAN_STATUS_TXABT    = 0b00010000,
    CAN_STATUS_TXSRCALL = 0b00001000,
    CAN_STATUS_TXSRC0   = 0b00000100,
    CAN_STATUS_TXSRC1   = 0b00000010,
    CAN_STATUS_TXSRC2   = 0b00000001
};

enum CAN_STATUS_RX
{
    CAN_STATUS_ERROR         = 0b10000000,
    CAN_STATUS_RXINVALIDMSG  = 0b01000000,
    CAN_STATUS_RXOVERFLOW    = 0b00010000,
    CAN_STATUS_RXOVERFLOWALL = 0b00001000,
    CAN_STATUS_RXOVERFLOW0   = 0b00000100,
    CAN_STATUS_RXOVERFLOW1   = 0b00000010
};

//////////////////////////////////////////////////////////////////////


extern void myCANInitialize(word mask, word filter1, word filter2);


extern bool myCANSendMessage(word id, byte *data, byte length, enum CAN_STATUS_TX *warnings);

extern bool myCANSendMessage0(word id, byte* data, byte length, enum CAN_STATUS_TX *status);

extern bool myCANSendMessage1(word id, byte* data, byte length, enum CAN_STATUS_TX *status);

extern bool myCANSendMessage2(word id, byte* data, byte length, enum CAN_STATUS_TX *status);


extern bool myCANReceiveMessage(word *id, byte *data, byte *length, enum CAN_STATUS_RX *warnings);

extern bool myCANReceiveMessage0(word *id, byte *data, byte *length, enum CAN_STATUS_RX *status);

extern bool myCANReceiveMessage1(word *id, byte *data, byte *length, enum CAN_STATUS_RX *status);

#endif      // CANDRIVERS_H
