#include "servo.h"

#define ANGLE_STD   0
#define PWM(ang)    (byte) ((SERVO_RANGE-(signed int) (ang))*SERVO_RES/(2*(signed int) SERVO_RANGE))

struct_adcAnalog adcServo[N_SERVOS];
struct_pwmServo pwmServo[N_SERVOS];


//struct_servo servoSignal={0,0,0,0};
// float controlRequested=0.0;        // Dados do controlador

/*
void initController(signed int pos0)
{
    initPic();

    servo(pos0);

    controlRequested=(float) pos0;

	//Esperar 1 segundo
    wait(1000);
}

void controllerReady(void)
{
    timeTick=0;
}


unsigned char deadlineSucess(void)
{
    if (timeTick==1)    return DEADLINE_ERROR;

    return SUCESS;
}
*/


// Inicializao das estruturas de dados
//
void initServoData(void)
{
    byte srv;

    for (srv=0; srv<N_SERVOS; ++srv)  {
        pwmServo[srv].pwm=PWM(ANGLE_STD);
        pwmServo[srv].angle=(signed int) ANGLE_STD*100;
        pwmServo[srv].error_prev=0;

        adcServo[srv].vpotMin=0;
        adcServo[srv].vpotImp=0;
    }
}

// Funcao para aquisicao do sinal presente no potencimetro do servo
//
#define SLOPE      180       // Com ganho 2... 884  // Declive*10 da funo Angulo=m*voltage+b
#define ORIGIN     236       //               235  // Ordenada na origem
#define POS(volt) (ORIGIN - (signed int) ((word) (volt)*SLOPE/100));

// Funo bloqueante de processamento das medidas
//
void transferServoSensors(void)
{
    byte srv;
	signed int ang;

    for (srv=0; srv<N_SERVOS; ++srv)  {

        ang=POS(adcServo[srv].vpotMin);     // Clculo do ngulo associado  tenso medida
        if (ang<-127)     ang=-127;         // Truncagem do resultado de modo a caber numa varivel de 8 bits
        else if (ang>127) ang= 127;

        //ang=adcServo[srv].vpotMin;     // Clculo do ngulo associado  tenso medida

        sensors.servo[srv].position=(signed char) ang;      // Posio angular
        sensors.servo[srv].velocity=99;                     // Velocidade
        sensors.servo[srv].current =adcServo[srv].vpotImp;  // Corrente
    }
}



//#define KP  0.10  //0.17        // 0.10 ou 0.07 Para a gama -90 a 0 estes // 0.06 ou 0.06 -45=>+45 //  0.05    Destino: Binrio0
//#define KD  0.45  //0.90        // 0.15    0.00 valores so os ideais     // 0.06    0.00          //  0.00

void controller(void)
{
    byte srv;
    signed char posReference;
    signed int error, error_dif;

    sensors.sysStatus.reset=false;

    // Aplicao do controlador
    for (srv=0; srv<N_SERVOS; ++srv)  {

        // Em caso de pedido, repor os motores na home position
        if (actuators.servo[srv].position==RESET_POSITION)  {
            reinitServo(srv);
            sensors.sysStatus.reset=true;
        }

        // Determinao da posio desejada
        posReference=actuators.servo[srv].position;
        if (posReference<-SERVO_RANGE)       posReference=-SERVO_RANGE;
        else if (posReference>SERVO_RANGE)   posReference= SERVO_RANGE;

        // Erro actual
   	    error=(signed int) posReference - sensors.servo[srv].position;
        if (error>-3 && error<3)  error=0;

   	    // Erro diferencial
        error_dif=error-pwmServo[srv].error_prev;
   	    // Actualizao do erro anterior
        pwmServo[srv].error_prev=error;

        // Caso o controlador esteja desligado,
        // aplicar imediatamente a posio desejada
        if (actuators.control.kp==0 && actuators.control.kd==0)  {
            sensors.sysStatus.controlOn=false;
            pwmServo[srv].angle=(signed int) posReference*100;
        }
        // No caso do controlador ligado, aplicar compensao
        else  {
            sensors.sysStatus.controlOn=true;

            // Clculo da nova posio a aplicar
            pwmServo[srv].angle += error*actuators.control.kp;
            pwmServo[srv].angle += error_dif*actuators.control.kd;
        
       	    // Limitao da posio a solicitar
            if (pwmServo[srv].angle<-(signed int) SERVO_RANGE*100)
                pwmServo[srv].angle=-(signed int) SERVO_RANGE*100;
       	    else if (pwmServo[srv].angle>(signed int) SERVO_RANGE*100)
                pwmServo[srv].angle= (signed int) SERVO_RANGE*100;
        }

        // Converso de posio para PWM
        pwmServo[srv].pwm=PWM((signed char) (pwmServo[srv].angle/100));
    }
}
