#include "servo.h"

// ngulo inicial a assumir pelos servos, e macro conversora de ngulo para unidades de PWM
#define ANGLE_STD   0
#define PWM(ang)    (byte) ((SERVO_RANGE-(signed int) (ang))*SERVO_RES/(2*(signed int) SERVO_RANGE))

// Estruturas externas
struct_adcAnalog adcServo[N_SERVOS];    // Dados sensoriais dos servos
struct_pwmServo pwmServo[N_SERVOS];     // Dados dos actuadores


// Inicializao das estruturas de dados
//
void initServoData(void)
{
    byte srv;

    for (srv=0; srv<N_SERVOS; ++srv)  {
        pwmServo[srv].pwm=PWM(ANGLE_STD);
        pwmServo[srv].angle=(signed int) ANGLE_STD*ANGLE_FACTOR;
        pwmServo[srv].error_prev=0;

        adcServo[srv].vpotMin=0;
        adcServo[srv].vpotImp=0;
    }
}

// Funcao para aquisicao do sinal presente no potencimetro do servo
//
// Os seguintes valores so utilizados para converter uma tenso entre 0 e 5000 num ngulo em graus.
// POS(graus)=ORIGIN-<tenso>*SLOPE/QUOCIENT
#define SLOPE      9        // Declive=1800     => Simplificando 1800/10000=9/50: SLOPE=9
#define QUOCIENT   50       // Quociente=10000  => Simplificando 1800/10000=9/50: QUOCIENT=50
#define ORIGIN     236      // Ordenada na origem
#define POS(volt) (ORIGIN - (signed int) ((word) (volt)*SLOPE/QUOCIENT));

// Funo bloqueante de processamento das medidas
//
void transferServoSensors(void)
{
    byte srv;
	signed int ang;

    for (srv=0; srv<N_SERVOS; ++srv)  {

        ang=POS(adcServo[srv].vpotMin);     // Clculo do ngulo associado  tenso medida
        if (ang<-127)     ang=-127;         // Truncagem do resultado de modo a caber numa varivel de 8 bits
        else if (ang>127) ang= 127;

        //ang=adcServo[srv].vpotMin;     // Clculo do ngulo associado  tenso medida

        sensors.servo[srv].position=(signed char) ang;      // Posio angular
        sensors.servo[srv].velocity=99;                     // Velocidade
        sensors.servo[srv].current =adcServo[srv].vpotImp;  // Corrente
    }
}



//#define KP  0.10  //0.17        // 0.10 ou 0.07 Para a gama -90 a 0 estes // 0.06 ou 0.06 -45=>+45 //  0.05    Destino: Binrio0
//#define KD  0.45  //0.90        // 0.15    0.00 valores so os ideais     // 0.06    0.00          //  0.00

void controller(void)
{
    bool controlOn;
    byte srv;
    signed char posReference;
    signed int error, error_dif;

    sensors.sysStatus.reset=false;
    
    // Aplicao do controlador
    for (srv=0; srv<N_SERVOS; ++srv)  {

        // Verificao do estado de RESET
        if (actuators.servo[srv].position==RESET_POSITION)  {
            reinitServo(srv);
            sensors.sysStatus.reset=true;
        }

        // Determinao da posio desejada (referncia)
        posReference=actuators.servo[srv].position;
        if (posReference<-SERVO_RANGE)       posReference=-SERVO_RANGE;
        else if (posReference>SERVO_RANGE)   posReference= SERVO_RANGE;

        // Erro actual
   	    error=(signed int) posReference - sensors.servo[srv].position;
        if (error>-2 && error<2)  error=0;
   	    // Erro diferencial
        error_dif=error-pwmServo[srv].error_prev;
   	    // Actualizao do erro anterior
        pwmServo[srv].error_prev=error;

        // No caso do controlador ligado, aplicar compensao
        if ((sensors.sysStatus.controlOn[srv] = actuators.control[srv].controlOn
            && (actuators.control[srv].kp!=0 || actuators.control[srv].kd!=0 || actuators.control[srv].ki!=0)))  {

            // Clculo da nova posio a aplicar
            pwmServo[srv].angle += error*actuators.control[srv].kp;         // Compensao proporcional
            pwmServo[srv].angle += error_dif*actuators.control[srv].kd;     // Compensao derivativa
        
       	    // Limitao da posio a solicitar
            if (pwmServo[srv].angle<-(signed int) SERVO_RANGE*ANGLE_FACTOR)
                pwmServo[srv].angle=-(signed int) SERVO_RANGE*ANGLE_FACTOR;
       	    else if (pwmServo[srv].angle>(signed int) SERVO_RANGE*ANGLE_FACTOR)
                pwmServo[srv].angle= (signed int) SERVO_RANGE*ANGLE_FACTOR;
        }
        // Caso o controlador esteja desligado,
        // aplicar imediatamente a posio desejada
        else
            pwmServo[srv].angle=(signed int) posReference*ANGLE_FACTOR;


        // Converso de posio para PWM
        pwmServo[srv].pwm=PWM((signed char) (pwmServo[srv].angle/ANGLE_FACTOR));
    }
}
