/*=======================================================================+
|             DEPARTAMENTO MECANICA UNIVERSIDADE AVEIRO                  |
|               Projecto Humanoide (PHUA - MODULROB)                     |
|                                                                        |
| Programadores:                                                         |
|              David Manuel Costa Gameiro          N:20281               |
|              Filipe Carvalho Mostardinha         N:22085               |
|                                                                        |
| Nome Ficheiro: Tipos.h                                                 |
|                                                                        |
|                            HEADER FILE                                 |
+=======================================================================*/


#ifndef _GLOBAL_H
#define _GLOBAL_H

#include <p18f258.h>
#include "types.h"

#define HOST_MCU    0                   // ID do MCU
#define HOST_SCU   (PORTC & 0b1111)     // ID deste SCU

// Frequncia de funcionamento do CPU (MHz) - 1/4 Fosc
#define F_CPU	10

// Conjuntos de sensores (identificado pelo bit 0bxxxxxx1x
#define SENSORS_REACTION    0
#define SENSORS_ORIENTATION 1

// Dados sobre o SCU
#define N_SERVOS        3                                   // Nmero de servos por SCU
#define N_FORCE_SENSORS 4                                   // Nmero de sensores de fora por p
#define SPECIAL_SENSORS true                                // Existncia de sensores especiais
#define SENSORS_ID      SENSORS_REACTION                    // IDentificao do tipo de sensores
#define FOOT_ID         0                                   // Identificao do p (se for o caso)

// Posies standard dos servos
#define HOME_POSITIONS { 0, 0, 0}
extern const signed char homePosition[N_SERVOS];
// Quando todas as posies desejadas possuem este valor, a junta  reinicializada
#define RESET_POSITION  -128

// Estrutura descritiva de um servo
typedef struct {
    signed char position;                                   // Posio
    signed char velocity;                                   // Velocidade
    unsigned char current;                                  // Corrente consumida
} struct_servo;

// Estrutura descritiva dos sensores
typedef struct {
    struct {
        bool reset;                                         // Motores nas home positions
        bool deadlineError;                                 // Violao de deadline
        bool motionFin[N_SERVOS];                           // Movimento terminado
        bool controlOn[N_SERVOS];                           // Estado on/off dos controladores
    } sysStatus;                                            // Estado do sistema
    struct_servo servo[N_SERVOS];                           // Sensores dos servos
    unsigned char reaction[N_FORCE_SENSORS];                // Sensores de fora dos ps
    signed char giroscope[2];                               // Giroscpio
    signed char inclinometer[2];                            // Inclinmetro
} struct_sensors;

typedef struct {
    struct {
        byte kp, kd, ki;
        bool controlOn;
    } control[N_SERVOS];
    struct_servo servo[N_SERVOS];
} struct_actuators;

// Variveis globais descritivas do sistema
extern volatile struct_sensors   sensors;                   // Sensores
extern volatile struct_actuators actuators;                 // Actuadores


extern void initGlobal(void);

extern void initSensors(void);

extern void initActuators(void);

extern void reinitServo(byte srv);

#endif
