#ifndef _SERVO_H
#define _SERVO_H

#include "global.h"

// Tick indicador da passagem dos 20ms
extern bool timeTick;

// Durao do Tick em milisegundos
#define TICK_TIME   20

// Limiar de deciso para deteco do impulso de corrente acima
// do valor mnimo:
#define VPOT_IMPULSE_LIMIAR	100		// 100/1023*5.0 = 0.488V < 1.0V
#define DUTY_MIN	1000  // 935  // CUIDADO COM ESTE VALOR: O perodo do Timer2 no deve fazer overflow!
#define DUTY_MAX	2000  // 2005

// Propriedades de funcionamento do servo
#define SERVO_RANGE	90              // Gama de funcionamento do servo para cada lado (graus)
#define SERVO_RES   180             // Quantidade de posies possveis: para 90 -> Resoluo=2 (2*RANGE/RES)

// Os seguintes valores so utilizados para converter uma tenso entre 0 e 5000 num ngulo em graus.
// POS(graus)=ORIGIN-<tenso>*SLOPE/QUOCIENT
#define SLOPE      12               // Declive=1800     => Simplificando 1800/10000=9/50: SLOPE=9
#define QUOCIENT   66               // Quociente=10000  => Simplificando 1800/10000=9/50: QUOCIENT=50
#define ORIGIN_STD {240, 240, 240}  // Valores standard do ponto de calibrao
extern signed int origin[N_SERVOS];
#define ORIGIN(volt,srv)    ((signed int) actuators.servo[srv].position + (signed int) ((word) (volt)*SLOPE/QUOCIENT))
#define POS(volt,srv)       (origin[srv] - (signed int) ((word) (volt)*SLOPE/QUOCIENT));

// Divisor a aplicar aos parmetros do controlador
#define ANGLE_FACTOR    100

// Pinos de interface do PIC
#define SERVO1_PIN	PORTBbits.RB1   // Saida de PWM para o servomotor (perodo de 20ms)
#define SERVO2_PIN	PORTBbits.RB4   // Saida de PWM para o servomotor
#define SERVO3_PIN	PORTBbits.RB5   // Saida de PWM para o servomotor

// Endereo de cada motor no multiplexer
#define MUX_INPUT(servo)    ((N_SERVOS-1-(servo)) & 0b11)

// Estrutura utilizada para calibrao automtica do ponto zero dos servos
#define N_MEASURES_CALIB    12     // Nmero de medies a efectuar para calcular a mdia (cuidado com o overflow)
typedef struct {
    bool med_calculated;           // Mdia calculada
    byte n_measure;                // Nmero de medidas efectuadas
    word voltage_sum;              // Soma da voltagem medida
    word voltage_med;              // Voltagem mdia
    word voltage_now;              // Valor actual da voltagem medida
} struct_calib;
extern struct_calib calibServo[N_SERVOS];

// Estrutura com dados necessrios  medio de velocidade
#define N_POS   5                   // Nmero de medies a armazenar no array de posies
typedef struct {
    byte        current[N_POS];     // Array com os ltimos valores de corrente
    signed char positions[N_POS];   // Array com as ltimas medies de posio
    signed char oldPosition;        // ndice do array de posies onde est armazenado a medio mais antiga
} struct_velocity;
extern volatile struct_velocity velocityServo[N_SERVOS];


// Estrutura com os dados a aplicar nos actuadores
typedef struct  {
    bool pwmOn;
    signed char applyPosition;      // Nova posio angular a aplicar
    byte pwm;                       // PWM associado  posio solicitada
    struct {
        double angle;               // ngulo a solicitar
        double deltaD_prev;         // Variao derivativa do ciclo anterior
        signed int  error_prev;
        signed char y_prev;         // Sinal de feedback (potencimetro) do ciclo anterior
        signed char y_prev2;        // Erro de h 2 ciclos atrs entre a posio efectiva e a desejada
    } control;
} struct_pwmServo;
extern volatile struct_pwmServo pwmServo[N_SERVOS];


extern void initServoData(void);

// Inicializao do Controlador
extern void initController(void);

// *****************************
// Funes para gerar um atraso
// *****************************

// Funcao para gerar um atraso (ms)
//
extern void wait(unsigned int value);

// Aplicao de uma nova velocidade a um servo
extern signed char servoPos(signed char position, byte srv);

// Activao dos sinais de PWM
extern void activatePWM(void);

//extern void transferServoSensors(word vpotMin, byte vpotImp, byte srv);

extern void controller(void);

#endif
