/*=======================================================================+
| DEPARTAMENTO MECANICA + DEPARTAMENTO DE ELECTRNICA E TELECOMUNICAES |
|                       DA UNIVERSIDADE AVEIRO                           |
|               Projecto Humanoide (PHUA - MODULROB)                     |
|                                                                        |
| Programadores:                                                         |
|              Milton Ruas da Silva                N:21824 (EET)         |
|              David Manuel Costa Gameiro          N:20281 (DEM)         |
|              Filipe Carvalho Mostardinha         N:22085 (DEM)         |
|                                                                        |
| Nome Ficheiro: global.h                                                |
|                                                                        |
|                            HEADER FILE                                 |
+=======================================================================*/

#ifndef _GLOBAL_H
#define _GLOBAL_H

#include <p18f258.h>
#include "types.h"

// _____ ESTADO DE FUNCIONAMENTO ESTTICO DO SISTEMA __________________________________________________________________

// Endereos
#define HOST_MCU    0                   // ID da unidade Master
#define HOST_SCU   (PORTC & 0b1111)     // ID deste SCU (Valor presente no PORTC)

// Conjuntos de sensores
typedef enum {
    SENSORS_REACTION=0,                 // Sensores de fora
    SENSORS_ORIENTATION=1               // SEnsores de orientao
} sensors_enum;

// Dados sobre o SCU
#define N_SERVOS        3                                   // Nmero de servos por SCU
#define N_FORCE_SENSORS 4                                   // Nmero de sensores de fora por p
#define SPECIAL_SENSORS false                               // Existncia de sensores especiais (fora/orientao)
#define SENSORS_ID      SENSORS_REACTION                    // IDentificao do tipo de sensores especiais
#define FOOT_ID         0                                   // Identificao do p (se for o caso de sensores de fora)

// Posies standard dos servos
#define HOME_POSITIONS { 0, 0, 0}
extern const signed char homePosition[N_SERVOS];

// _____ ESTRUTURAS GLOBAIS DO SISTEMA ________________________________________________________________________________

// Estrutura descritiva do estado de um servo
typedef struct {
    signed char position;                                   // Posio
    signed char velocity;                                   // Velocidade
    unsigned char current;                                  // Corrente consumida
} struct_servo;

// Estrutura descritiva dos sensores
typedef struct {
    struct {
        bool reset;                                         // Motores nas home positions
        bool deadlineError;                                 // Violao de deadline
        bool motionFin[N_SERVOS];                           // Movimento terminado
        bool controlOn[N_SERVOS];                           // Estado on/off dos controladores PID
    } sysStatus;                                         // Estado do sistema
    struct_servo servo[N_SERVOS];                           // Sensores dos servos (posio, velocidade e corrente)
    unsigned char reaction[N_FORCE_SENSORS];                // Sensores de fora dos ps
    signed char giroscope[2];                               // Giroscpio
    signed char inclinometer[2];                            // Inclinmetro
} struct_sensors;

// Estrutura descritiva dos actuadores
typedef struct {
    struct {
        byte kp, kd, ki;                                    // Parmetros Kp, Kd e Ki
        bool controlOn;                                     // Controlador On/Off?
    } control[N_SERVOS];                                 // Controlador PID
    struct_servo servo[N_SERVOS];                           // Informao de actuao (posio, velocidade e corrente)
} struct_actuators;

// Variveis globais descritivas do sistema
extern volatile struct_sensors   sensors;                   // Sensores
extern volatile struct_actuators actuators;                 // Actuadores

// _____ FUNES DE INICIALIZAO DO SISTEMA __________________________________________________________________________

// Inicializao de todas as estruturas globais
extern void initGlobal(void);

// Inicializao da estrutura sensores
extern void initSensors(void);

// Inicializao da estrutura actuadores
extern void initActuators(void);

// Reinicializao da actuao dos servos
extern void reinitServo(byte srv);

#endif
