/*-----------------------------------------------------------------------+
|             DEPARTAMENTO MECANICA UNIVERSIDADE AVEIRO                  |
|                                                                        |
| Nome:                                                                  |
|       David Manuel Costa Gameiro          N:20281                      |
|       Filipe Carvalho Mostardinha         N:22085                      |
|                                                                        |
| Data:   10/Maio/2004     Nome Ficheiro: InitCan.c                      |
+-----------------------------------------------------------------------*/

#include "can.h"
#include "CanDrivers.h"

// Comprimento das mensagens CAN (bytes)
#define CAN_MESSAGE_LENGTH   8

// Alguns bytes das mensagens CAN (de recepo)
#define CAN_TYPE    0
#define CAN_DATA1   1

// _____ IDENTIFICADOR DAS MENSAGENS CAN ______________________________________________________
// Constantes de referncia ao identificador das mensagens CAN
enum MESSAGE_IDENTIFIER  {
    CANINDEX_MASK  = 0b10000000000,
    XCUDST_MASK    = 0b01111000000,
    XCUSRC_MASK    = 0b00000111100,
    OPERATION_MASK = 0b00000000011,
    CAN_MASK       = CANINDEX_MASK | XCUDST_MASK,

    CANINDEX_LOC   =10,
    XCUDST_LOC     = 6,
    XCUSRC_LOC     = 2,
    OPERATION_LOC  = 0,

    OP_UPDATE_SENSORS   = 0b00,
    OP_UPDATE_ACTUATORS = 0b01,
    OP_SLAVECAN_STATUS  = 0b10
};

// Extrao de valores do identificador
#define canIndex(x)  (((unsigned int) (x) & CANINDEX_MASK ) >> CANINDEX_LOC )
#define xcuDST(x)    (((unsigned int) (x) & XCUDST_MASK   ) >> XCUDST_LOC   )
#define xcuSRC(x)    (((unsigned int) (x) & XCUSRC_MASK   ) >> XCUSRC_LOC   )
#define operation(x) (((unsigned int) (x) & OPERATION_MASK) >> OPERATION_LOC)

// Construo de padres do identificador
#define canIndex_patt(y)  (((unsigned int) (y) << CANINDEX_LOC ) & CANINDEX_MASK )
#define xcuDST_patt(y)    (((unsigned int) (y) << XCUDST_LOC   ) & XCUDST_MASK   )
#define xcuSRC_patt(y)    (((unsigned int) (y) << XCUSRC_LOC   ) & XCUSRC_MASK   )
#define operation_patt(y) (((unsigned int) (y) << OPERATION_LOC) & OPERATION_MASK)

// Construo do identificador
#define filter(ind)               (canIndex_patt(ind) | xcuDST_patt(HOST_SCU))
#define identifierMSG(op,ind)     (canIndex_patt(ind) | xcuDST_patt(HOST_MCU) | xcuSRC_patt(HOST_SCU) | operation_patt(op))
// ___________________________________________________________________________________________

// PACOTE DE TIPO DAS MENSAGENS OP_UPDATE_SENSORS ____________________________________________
// Constantes de referncia ao byte de tipo dos pacotes de recepo
enum MESSAGE_UPDATE_SENSORS  {
    // Mscaras para extrao de um campo
    SCUSRC_MASK   = 0b11110000,
    MSGINDEX_MASK = 0b00001000,
    SPECIAL_MASK  = 0b00000100,
    SENSORS_MASK  = 0b00000010,
    FOOTID_MASK   = 0b00000001,

    // Localizao dos campos no byte de tipo
    SCUSRC_LOC    = 4,
    MSGINDEX_LOC  = 3,
    SPECIAL_LOC   = 2,
    SENSORS_LOC   = 1,
    FOOTID_LOC    = 0,

    // Identificao do pacote
    MSGINDEX_1  = 0,
    MSGINDEX_2 = 1
};

// Macros para leitura dos diversos campos no byte de tipo
#define scuSRC_patt(y)   (byte) (((unsigned int) (y) << SCUSRC_LOC  ) & SCUSRC_MASK  )
#define msgIndex_patt(y) (byte) (((unsigned int) (y) << MSGINDEX_LOC) & MSGINDEX_MASK)
#define special_patt(y)  (byte) (((unsigned int) (y) << SPECIAL_LOC ) & SPECIAL_MASK )
#define sensors_patt(y)  (byte) (((unsigned int) (y) << SENSORS_LOC ) & SENSORS_MASK )
#define footID_patt(y)   (byte) (((unsigned int) (y) << FOOTID_LOC  ) & FOOTID_MASK  )

// Macros para construo do byte de tipo
#define typePacket(pack_id,sp,sens,foot) (scuSRC_patt(HOST_SCU) | msgIndex_patt(pack_id) | special_patt(sp) | sensors_patt(sens) | footID_patt(foot))
// ___________________________________________________________________________________________________________

// _____ PACOTE DE ESTADO DO SISTEMA _________________________________________________________________________
// Pacote de estado do sistema (ltimo byte das mensagens OP_READ_JOINT
enum MESSAGE_SYSSTATUS_PACKET {
    // Mscaras dos bits
    DEADLINEERROR_MASK = 0b00000100,     // Ocorrncia de violao de deadline
    CONTROLON_MASK     = 0b00000010,     // Controlador PID ligado
    RESET_MASK         = 0b00000001,     // Motores nas home positions

    // Localizao dos bits
    DEADLINEERROR_LOC  = 2,
    CONTROLON_LOC      = 1,
    RESET_LOC          = 0
};

// Construes dos padres do pacote de estado do sistema
#define deadlineError_patt(d) (byte) (((unsigned int) (d) << DEADLINEERROR_LOC) & DEADLINEERROR_MASK)
#define controlOn_patt(c)     (byte) (((unsigned int) (c) << CONTROLON_LOC    ) & CONTROLON_MASK    )
#define reset_patt(r)         (byte) (((unsigned int) (r) << RESET_LOC        ) & RESET_MASK        )

// Construo do pacote de estado do sistema
#define sysStatusPacket(res,cOn,d)   (reset_patt(res) | controlOn_patt(cOn) | deadlineError_patt(d))
// _________________________________________________________________________________________________________

// _____ PACOTE DE TIPO DAS MENSAGENS OP_SLAVECAN_STATUS _____________________________________________
// Estados de erro relativos ao CAN
enum CAN_ERRORS  {
    CAN_ERROR           = 0b00000001,
    RX_INVALIDMSG_ERROR = 0b00000010,       // Mensagem invlida
    RX_OVERFLOW_ERROR   = 0b00000100,       // Erro de overflow dos buffers de recepo
    RX_INVALID_LENGTH   = 0b00001000,       // Comprimento invlido
    RX_INVALID_DATA     = 0b00010000,       // Dados da mensagem invlidos
    RX_UNKNOWN_ERROR    = 0b00100000,       // Erro desconhecido
    TX_NOTPOSSIBLE      = 0b01000000,       // Transmisso no possvel devido  ocupao de todos os buffers
    TX_ERROR            = 0b10000000,

    TX_BITS = 0b11000000,                   // Bits relativos  transmisso
    RX_BITS = 0b00111111                    // Bits relativos  recepo
};
// ___________________________________________________________________________________________________

// Estrutura descritiva dos erros ocorridos
struct  {
    enum CAN_ERRORS flags;          // Causas
    byte tx_count, rx_count;        // Nmero de ocorrncias
} canErrors={false,0,0};


static bool sendCanStatus(void);                    // Transmisso de uma mensagem de erro
static void updateServoSensors(byte *message);
static void updateSpecialSensors(byte *message);
static void updateActuators(byte *can_message);     // Actualizao dos dados sensoriais do sistema
static void setError(enum CAN_ERRORS error);        // Registo de um erro ocorrido com o CAN


// Mascara autorizando a que o filtro verifique apenas os 4 bits menos
// significativos (SCU ID)
#define MASCARA XCUDST_MASK

// Inicializao do CAN
//
void initCan(void)
{
    // Programacao dos pinos RB3 e RB2 como entrada e sada respectivamente
	TRISB=(TRISB | 0x08) & 0xFB;

    // Inicializao dos registos do CAN
    myCANInitialize(MASCARA,filter(0),filter(1));

    // Interrupes (recepo) de baixa prioridade
    IPR3bits.RXB0IP=0;
    IPR3bits.RXB1IP=0;
    // Activa os interrupts dos buffers recepcao CAN
    PIE3bits.RXB0IE=0;                                  // Interrupts Enable
    PIE3bits.RXB1IE=0;
}


// Transmisso de uma mensagem CAN relativa aos actuadores do sistema
//
bool sendCanMessage(void)
{
    bool res1, res2, res3;
    word identifier;
    byte message1[CAN_MESSAGE_LENGTH], message2[CAN_MESSAGE_LENGTH];
    enum CAN_STATUS_TX warnings1, warnings2;

    // Construo das duas mensagens com o contedo sensorial
    updateServoSensors(message1);
    updateSpecialSensors(message2);

    // Identificador a utilizar nas mensagens a enviar
    identifier=identifierMSG(OP_UPDATE_SENSORS,0);
    //funao que envia mensagens
    if (!(res1=myCANSendMessage(identifier, message1, CAN_MESSAGE_LENGTH, &warnings1)))
        setError(TX_NOTPOSSIBLE);

    // Identificador a utilizar nas mensagens a enviar
    identifier=identifierMSG(OP_UPDATE_SENSORS,1);
    if (!(res2=myCANSendMessage(identifier, message2, CAN_MESSAGE_LENGTH, &warnings2)))
        setError(TX_NOTPOSSIBLE);

    // Verificao de erros da ltima transmisso
    if ((warnings1 | warnings2) & CAN_STATUS_ERROR)  setError(CAN_ERROR);
    if ((warnings1 | warnings2) & CAN_STATUS_TXERR)  setError(TX_ERROR);

    // Caso tenha ocorrido algum erro no passado, transmitir
    // mensagem extra com informaes detalhadas
    if (canErrors.flags && (res3=sendCanStatus()))  canErrors.flags=0;

    // Apenas retorna true, se as duas transmisses tiveram sucesso!
    return (res1 && res2 && res3);
}


static bool sendCanStatus(void)
{
    bool res;
    byte i, message[CAN_MESSAGE_LENGTH];
    word identifier;
    enum CAN_STATUS_TX warnings;

    // Identificador da mensagem
    // Esta mensagem tem ndice 1 para lhe dar baixa prioridade no bus
    identifier=identifierMSG(OP_SLAVECAN_STATUS,1);

    // Preenchimento de um array com a informao dos erros ocorridos para transmisso
    message[0]=canErrors.flags;         // Flags com a identificao dos erros ocorridos
    message[1]=canErrors.tx_count;      // Nmero de erros de transmisso
    message[2]=canErrors.rx_count;      // Nmero de erros de recepo
    message[3]=HOST_SCU;                // Identificao do SCU fonte

    // Finalizao do preenchimento com zeros
    for (i=4; i<CAN_MESSAGE_LENGTH; ++i)  message[i]=0;

    // Transmisso da mensagem
    if (!(res=myCANSendMessage(identifier, message, CAN_MESSAGE_LENGTH, &warnings)));
        // Caso ocorra um erro, assinal-lo
        setError(TX_NOTPOSSIBLE);

    // Verificao de erros da ltima transmisso
    if (warnings & CAN_STATUS_ERROR)  setError(CAN_ERROR);
    if (warnings & CAN_STATUS_TXERR)  setError(TX_ERROR);

    return res;
}


// ****************************************************************************
// Recepo de uma mensagem CAN de um dos SCU
//
bool receiveCanMessage(void)
{
    bool res;                           // Resultado da recepo
    word id;
    byte length;
    enum CAN_STATUS_RX warnings;
    byte message[CAN_MESSAGE_LENGTH];

    // Um erro ocorreu na recepo
    res=myCANReceiveMessage(&id, message, &length, &warnings);

    // Erro no Barramento CAN
    if (warnings & CAN_STATUS_ERROR)         setError(CAN_ERROR);
    // Erro de Overflow
    if (warnings & CAN_STATUS_RXOVERFLOW)    setError(RX_OVERFLOW_ERROR);
    // Erro de mensagem invlida
    if (warnings & CAN_STATUS_RXINVALIDMSG)  setError(RX_INVALIDMSG_ERROR);

    // Se a recepo no foi realizada, abandonar funo!
    if (!res)  return false;

    // Comprimento de dados invlido
    if (length<CAN_MESSAGE_LENGTH)
        setError(RX_INVALID_LENGTH);
    // Tudo OK!
    else if (operation(id)==OP_UPDATE_ACTUATORS)
        updateActuators(message);         // Actualizao dos actuadores
    else
        setError(RX_UNKNOWN_ERROR);  // Erro: operao no vlida!

    return true;
}


// ****************************************************************************
// Registo de um erro ocorrido com o CAN
//
static void setError(enum CAN_ERRORS error)
{
    // Erros ocorridos
    canErrors.flags|=error;
    
    // Actualizao dos contadores de erros
    if (canErrors.tx_count<0xFF && (error & TX_BITS))  canErrors.tx_count++;
    if (canErrors.rx_count<0xFF && (error & RX_BITS))  canErrors.rx_count++;
}


// ****************************************************************************
// Actualizao dos dados sensoriais do sistema
//
static void updateActuators(byte *can_message)
{
    byte i;

    // Actualizar as posies e velocidades dos servos da junta
    for (i=0; i<N_SERVOS; ++i)  {
        actuators.servo[i].position=can_message[i];
        actuators.servo[i].velocity=can_message[i+N_SERVOS];
    }
    actuators.control.kp=can_message[2*N_SERVOS+0];
    actuators.control.kd=can_message[2*N_SERVOS+1];

    // REMOVER
    for (i=0; i<N_SERVOS; ++i)  {
        sensors.servo[i].position=actuators.servo[i].position;
        sensors.servo[i].velocity=actuators.servo[i].velocity;
    }
    sensors.sysStatus.controlOn = (actuators.control.kp==0 && actuators.control.kd==0) ? false : true;

    for (i=0; i<N_SERVOS && actuators.servo[i].position==-128; ++i);
    sensors.sysStatus.reset = (i==N_SERVOS) ? true : false;
    // REMOVER
}

static void updateServoSensors(byte *message)
{
    byte i, ind=0;

    // Preenchimento do primeiro pacote
    // Byte de tipo
    message[ind++]=typePacket(MSGINDEX_1,0,0,0);
    for (i=0; i<N_SERVOS; ++i,++ind)  {
        message[ind]=sensors.servo[i].position;
        message[N_SERVOS+ind]=sensors.servo[i].velocity;
    }

    ind+=N_SERVOS;      // Actualizao do ndice da mensagem CAN tendo em conta as velocidades introduzidas
    // Pacote de estado do sistema
    message[ind++]=sysStatusPacket(sensors.sysStatus.reset,sensors.sysStatus.controlOn,sensors.sysStatus.deadlineError);

    // Finalizao do preenchimento com zeros
    while (ind<CAN_MESSAGE_LENGTH)  message[ind++]=0;
}

static void updateSpecialSensors(byte *message)
{
    byte i, ind=0;

    // Preenchimento do segundo pacote
    // Byte de tipo
    message[ind++]=typePacket(MSGINDEX_2,SPECIAL_SENSORS,SENSORS_ID,FOOT_ID);
    // Dados das correntes consumidas pelos servos
    for (i=0; i<N_SERVOS; ++i,++ind)
        message[ind]=sensors.servo[i].current;
    // Se h sensores especiais
    if (SPECIAL_SENSORS)  {
        // No caso de sensores de reaco...
        if (SENSORS_ID==SENSORS_REACTION)
            for (i=0; i<N_FORCE_SENSORS; ++i,++ind)
                message[ind]=sensors.reaction[i];
        else  {
            // Inclinmetro/Giroscpio
            message[ind++]=sensors.inclinometer[0];
            message[ind++]=sensors.inclinometer[1];
            message[ind++]=sensors.giroscope[0];
            message[ind++]=sensors.giroscope[1];
        }
    }
    // Finalizao do preenchimento com zeros
    while (ind<CAN_MESSAGE_LENGTH)  message[ind++]=0;
}
